#######################################################################
#                                                                      
#         Copyright 2015-2016 by Ullrich Koethe and Philip Schill      
#                                                                      
#    This file is part of the VIGRA computer vision library.           
#    The VIGRA Website is                                              
#        http://hci.iwr.uni-heidelberg.de/vigra/                       
#    Please direct questions, bug reports, and contributions to        
#        ullrich.koethe@iwr.uni-heidelberg.de    or                    
#        vigra@informatik.uni-hamburg.de                               
#                                                                      
#    Permission is hereby granted, free of charge, to any person       
#    obtaining a copy of this software and associated documentation    
#    files (the "Software"), to deal in the Software without           
#    restriction, including without limitation the rights to use,      
#    copy, modify, merge, publish, distribute, sublicense, and/or      
#    sell copies of the Software, and to permit persons to whom the    
#    Software is furnished to do so, subject to the following          
#    conditions:                                                       
#                                                                      
#    The above copyright notice and this permission notice shall be    
#    included in all copies or substantial portions of the             
#    Software.                                                         
#                                                                      
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND    
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES   
#    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND          
#    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT       
#    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,      
#    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING      
#    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR     
#    OTHER DEALINGS IN THE SOFTWARE.                                   
#                                                                      
#######################################################################

from __future__ import division, print_function
import sys
print("\nexecuting test file", __file__, file=sys.stderr)
exec(compile(open('set_paths.py', "rb").read(), 'set_paths.py', 'exec'))

from nose.tools import assert_equal, raises, assert_raises
import vigra
import numpy as np

def checkAboutSame(i1,i2):
    assert(i1.shape==i2.shape)
    difference=np.sum(np.abs(i1-i2))/float(np.size(i1))
    assert(difference<5)

def checkEqual(i1, i2):
    assert(i1.shape==i2.shape)
    assert((i1==i2).all())

def test_convexHull():
    points = np.array([[0, 0], [2, 0], [2, 1], [0, 1], [1, 0.5]], dtype=np.float32)
    res = np.array([[0, 0], [0, 1], [2, 1], [2, 0], [0, 0]], dtype=np.float32)
    res = vigra.taggedView(res)
    hull = vigra.geometry.convexHull(points)
    checkAboutSame(hull, res)
    hull = vigra.geometry.convexHull(vigra.taggedView(points))
    checkAboutSame(res, hull)
    assert_raises(ValueError, vigra.geometry.convexHull, points.transpose())
    assert_raises(ValueError, vigra.geometry.convexHull, points.astype(np.uint8))
    assert_raises(ValueError, vigra.geometry.convexHull, 0, "a")

def test_convolveOneDimension():
    im = np.array([[0, 0, 0, 0, 0],
                   [0, 0, 1, 0, 0],
                   [0, 0, 0, 0, 0]], dtype=np.float64)
    imc = np.array([[0, 0, 0, 0, 0],
                    [0, 1, 2, 3, 0],
                    [0, 0, 0, 0, 0]], dtype=np.float64)
    k = vigra.filters.explictKernel(-1, 1, np.array([1, 2, 3], dtype=np.float64))
    res = vigra.filters.convolveOneDimension(im, 0, k)
    checkAboutSame(res, imc)
    assert_raises(ValueError, vigra.filters.convolveOneDimension, im)
    assert_raises(ValueError, vigra.filters.convolveOneDimension, im, 0)
    assert_raises(ValueError, vigra.filters.convolveOneDimension, im.astype(np.uint8), 0, k)
    assert_raises(ValueError, vigra.filters.convolveOneDimension, [0, 1], 0, k)

def test_convolve():
    # Test convolve with a 2D kernel.
    im = np.array([[0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0],
                   [0, 0, 1, 0, 0],
                   [0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0]], dtype=np.float64)
    imc0 = np.array([[0, 0, 0, 0, 0],
                     [0, 1, 2, 3, 0],
                     [0, 4, 5, 6, 0],
                     [0, 7, 8, 9, 0],
                     [0, 0, 0, 0, 0]], dtype=np.float64)
    k = vigra.filters.Kernel2D()
    k.initExplicitly((-1, -1), (1, 1), np.array([[1, 2, 3],
                                                 [4, 5, 6],
                                                 [7, 8, 9]], dtype=np.float64))
    res = vigra.filters.convolve(im, k)
    checkAboutSame(res, imc0)
    assert_raises(ValueError, vigra.filters.convolve, im)
    assert_raises(ValueError, vigra.filters.convolve, im.astype(np.uint8), k)
    assert_raises(ValueError, vigra.filters.convolve, [0, 1], k)

    # Test convolve with a 1D kernel.
    imc1 = np.array([[0, 0, 0, 0, 0],
                     [0, 1, 2, 3, 0],
                     [0, 2, 4, 6, 0],
                     [0, 3, 6, 9, 0],
                     [0, 0, 0, 0, 0]], dtype=np.float64)
    k = vigra.filters.Kernel1D()
    k.initExplicitly(-1, 1, np.array([1, 2, 3], dtype=np.float64))
    res = vigra.filters.convolve(im, k)
    checkAboutSame(res, imc1)

    # Test convolve with two 1D kernels.
    imc2 = np.array([[0, 0, 0, 0, 0],
                     [0, 0.5, 1, 2, 0],
                     [0, 1, 2, 4, 0],
                     [0, 1.5, 3, 6, 0],
                     [0, 0, 0, 0, 0]], dtype=np.float64)
    k0 = vigra.filters.explictKernel(-1, 1, np.array([1, 2, 3], dtype=np.float64))
    k1 = vigra.filters.explictKernel(-1, 1, np.array([0.5, 1, 2], dtype=np.float64))
    res = vigra.filters.convolve(im, (k0, k1))
    checkAboutSame(res, imc2)

def test_gaussianSmoothing():
    im = np.array([[0, 0, 0],
                   [0, 1, 0],
                   [0, 0, 0]], dtype=np.float64)
    imc = np.array([[ 0.04532707,  0.16757448,  0.04532707],
                    [ 0.16757448,  0.61952398,  0.16757448],
                    [ 0.04532707,  0.16757448,  0.04532707]])
    res = vigra.filters.gaussianSmoothing(im, 0.5)
    checkAboutSame(res, imc)
    assert_raises(ValueError, vigra.filters.gaussianSmoothing, im)
    assert_raises(ValueError, vigra.filters.gaussianSmoothing, im.astype(np.int8), 0.5)

def test_laplacianOfGaussian():
    im = np.zeros((6, 5), dtype=np.float64)
    im[2, 2] = 1
    imc = np.array([[ 0.07002893,  0.08373281,  0.08667994,  0.08373281,  0.07002893],
                    [ 0.08373281,  0.0174964 , -0.08323153,  0.0174964 ,  0.08373281],
                    [ 0.08646211, -0.0837286 , -0.31618431, -0.0837286 ,  0.08646211],
                    [ 0.07846283,  0.00859282, -0.09564048,  0.00859282,  0.07846283],
                    [ 0.0352323 ,  0.04236348,  0.04414477,  0.04236348,  0.0352323 ],
                    [ 0.01053995,  0.01780715,  0.0248179 ,  0.01780715,  0.01053995]])
    res = vigra.filters.laplacianOfGaussian(im)
    checkAboutSame(res, imc)
    assert_raises(ValueError, vigra.filters.laplacianOfGaussian, im.astype(np.uint8))

def test_gaussianDivergence():
    im = np.zeros((5, 5, 2), dtype=np.float64)
    im[2, 2] = [0, 1]
    imc = np.array([[  0.,   2.47013247e-02,   5.96311195e-19, -2.47013247e-02,   0.],
                    [  0.,   5.63656325e-02,   1.35525272e-18, -5.63656325e-02,   0.],
                    [  0.,   9.12597369e-02,   2.16840434e-18, -9.12597369e-02,   0.],
                    [  0.,   5.63656325e-02,   1.35525272e-18, -5.63656325e-02,   0.],
                    [  0.,   2.47013247e-02,   5.96311195e-19, -2.47013247e-02,   0.]])
    res = vigra.filters.gaussianDivergence(im)
    checkAboutSame(res, imc)
    assert_raises(ValueError, vigra.filters.gaussianDivergence, np.zeros((5, 6, 3), dtype=np.float64))
    assert_raises(ValueError, vigra.filters.gaussianDivergence, np.zeros((10, 11, 12, 2), dtype=np.float64))

def test_multiBinaryErosionDilation():
    im = np.array([[0, 0, 0, 0, 0],
                   [0, 1, 1, 0, 0],
                   [0, 0, 1, 0, 0],
                   [0, 0, 0, 0, 0]], dtype=np.uint8)
    imc = np.array([[0, 1, 1, 0, 0],
                    [1, 1, 1, 1, 0],
                    [0, 1, 1, 1, 0],
                    [0, 0, 1, 0, 0]], dtype=np.uint8)
    imc2 = np.array([[1, 1, 1, 0, 0],
                    [1, 1, 1, 1, 0],
                    [0, 1, 1, 1, 0],
                    [0, 0, 1, 0, 0]], dtype=np.uint8)
    res = vigra.filters.multiBinaryErosion(imc, 1)
    checkEqual(res, im)
    assert_raises(ValueError, vigra.filters.multiBinaryErosion, imc)
    assert_raises(ValueError, vigra.filters.multiBinaryErosion, imc.astype(np.float64))
    res = vigra.filters.multiBinaryDilation(im, 1)
    checkEqual(res, imc)
    assert_raises(ValueError, vigra.filters.multiBinaryDilation, im)
    assert_raises(ValueError, vigra.filters.multiBinaryDilation, im.astype(np.float64))

    res = vigra.filters.multiBinaryOpening(im, 1)
    checkEqual(res, np.zeros((4, 5), dtype=np.uint8))
    res = vigra.filters.multiBinaryOpening(imc, 1)
    checkEqual(res, imc)
    assert_raises(ValueError, vigra.filters.multiBinaryOpening, im)
    assert_raises(ValueError, vigra.filters.multiBinaryOpening, im.astype(np.float64))
    res = vigra.filters.multiBinaryClosing(im, 1)
    checkEqual(res, im)
    res = vigra.filters.multiBinaryClosing(imc, 1)
    checkEqual(res, imc2)
    assert_raises(ValueError, vigra.filters.multiBinaryClosing, im)
    assert_raises(ValueError, vigra.filters.multiBinaryClosing, im.astype(np.float64))

def test_multiGrayscaleErosionDilation():
    im = np.array([[0, 0, 0, 0, 0],
                   [0, 128, 100, 0, 0],
                   [0, 0, 128, 0, 0],
                   [0, 0, 0, 0, 0]], dtype=np.uint8)
    imc0 = np.array([[0, 0, 0, 0, 0],
                     [0, 81, 81, 0, 0],
                     [0, 0, 81, 0, 0],
                     [0, 0, 0, 0, 0]], dtype=np.uint8)
    imc1 = np.array([[ 56,  92,  64,  28,   0],
                     [ 92, 128, 100,  64,   0],
                     [ 56,  92, 128,  92,   0],
                     [  0,  56,  92,  56,   0]], dtype=np.uint8)
    imc2 = np.array([[0, 47, 19, 0, 0],
                     [47, 128, 100, 19, 0],
                     [0, 47, 128, 47, 0],
                     [0, 0, 47, 0, 0]], dtype=np.uint8)
    res = vigra.filters.multiGrayscaleErosion(im, 9)
    checkEqual(res, imc0)
    assert_raises(ValueError, vigra.filters.multiGrayscaleErosion, im)
    assert_raises(ValueError, vigra.filters.multiGrayscaleErosion, im.astype(np.int16), 9)
    res = vigra.filters.multiGrayscaleDilation(im, 6)
    checkEqual(res, imc1)
    assert_raises(ValueError, vigra.filters.multiGrayscaleDilation, im)
    assert_raises(ValueError, vigra.filters.multiGrayscaleDilation, im.astype(np.int16), 6)

    res = vigra.filters.multiGrayscaleOpening(im, 9)
    checkEqual(res, imc0)
    assert_raises(ValueError, vigra.filters.multiGrayscaleOpening, im)
    assert_raises(ValueError, vigra.filters.multiGrayscaleOpening, im.astype(np.int16), 9)
    res = vigra.filters.multiGrayscaleClosing(im, 9)
    checkEqual(res, imc2)
    assert_raises(ValueError, vigra.filters.multiGrayscaleClosing, im)
    assert_raises(ValueError, vigra.filters.multiGrayscaleClosing, im.astype(np.int16), 9)

def test_distanceTransform():
    # Test distanceTransform and distanceTransform2D.
    im = np.array([[0, 0, 0, 0, 0],
                   [0, 1, 1, 0, 0],
                   [0, 0, 1, 0, 0],
                   [0, 0, 0, 0, 0]], dtype=np.uint32)
    s2 = 1.41421354
    s5 = 2.23606801
    imd = np.array([[s2, 1, 1, s2, s5],
                    [1, 0, 0, 1, 2],
                    [s2, 1, 0, 1, 2],
                    [s5, s2, 1, s2, s5]], dtype=np.float32)
    res = vigra.filters.distanceTransform(im)
    checkAboutSame(res, imd)
    assert_raises(ValueError, vigra.filters.distanceTransform, np.zeros((5, 6, 7, 8), dtype=np.float32))

    # Test vectorDistanceTransform.
    im = im.astype(np.float32)
    imd = np.array([[[1, 1], [1, 0], [1, 0], [1, -1], [1, -2]],
                    [[0, 1], [0, 0], [0, 0], [0, -1], [0, -2]],
                    [[-1, 1], [0, 1], [0, 0], [0, -1], [0, -2]],
                    [[-1, 2], [-1, 1], [-1, 0], [-1, -1], [-1, -2]]], dtype=np.float32)
    res = vigra.filters.vectorDistanceTransform(im)
    checkAboutSame(res, imd)
    assert_raises(ValueError, vigra.filters.vectorDistanceTransform, np.zeros((5, 6, 7, 8), dtype=np.float32))
    
def test_boundaryDistanceTransform():
    # Test boundaryDistanceTransform.
    im = np.array([[2, 2, 2, 2, 2, 2],
                   [2, 2, 2, 2, 2, 2],
                   [1, 1, 1, 1, 2, 2],
                   [1, 1, 1, 1, 1, 2],
                   [1, 1, 1, 1, 1, 1],
                   [1, 1, 1, 1, 1, 1]], dtype=np.float32)
    s2 = 0.91421354
    s3 = 1.73606801
    s5 = 2.32842708
    s6 = 3.10555124
    imd = np.array([[1.5, 1.5, 1.5, 1.5, s3, s5],
                    [0.5, 0.5, 0.5, 0.5, s2, s3],
                    [0.5, 0.5, 0.5, 0.5, 0.5, s2],
                    [1.5, 1.5, 1.5, s2, 0.5, 0.5],
                    [2.5, 2.5, s5, s3, s2, 0.5],
                    [3.5, 3.5, s6, s5, s3, 1.5]], dtype=np.float32)
    res = vigra.filters.boundaryDistanceTransform(im)
    checkAboutSame(res, imd)
    assert_raises(ValueError, vigra.filters.boundaryDistanceTransform, im.astype(np.uint8))
    assert_raises(ValueError, vigra.filters.boundaryDistanceTransform, np.zeros((5, 6, 7, 8), dtype=np.float32))

    # Test boundaryVectorDistanceTransform.
    imd2 = np.array([[[ 1.5,  0. ], [ 1.5,  0. ], [ 1.5,  0. ], [ 1.5,  0. ], [ 1.5, -1. ], [ 1.5, -2. ]],
                     [[ 0.5,  0. ], [ 0.5,  0. ], [ 0.5,  0. ], [ 0.5,  0. ], [ 0.5, -1. ], [ 1. , -1.5]],
                     [[-0.5,  0. ], [-0.5,  0. ], [-0.5,  0. ], [-0.5,  0. ], [ 0. , -0.5], [ 0.5, -1. ]],
                     [[-1.5,  0. ], [-1.5,  0. ], [-1.5,  0. ], [-1. ,  0.5], [-0.5,  0. ], [ 0. , -0.5]],
                     [[-2.5,  0. ], [-2.5,  0. ], [-2. ,  1.5], [-1.5,  1. ], [-1. ,  0.5], [-0.5,  0. ]],
                     [[-3.5,  0. ], [-3.5,  0. ], [-2.5,  2. ], [-2. ,  1.5], [-1.5,  1. ], [-1.5,  0. ]]],
                    dtype=np.float32)
    res = vigra.filters.boundaryVectorDistanceTransform(im)
    checkAboutSame(res, imd2)
    assert_raises(ValueError, vigra.filters.boundaryVectorDistanceTransform, im.astype(np.uint8))
    assert_raises(ValueError, vigra.filters.boundaryVectorDistanceTransform, np.zeros((5, 6, 7, 8), dtype=np.float32))

def test_eccentricityTransform():
    # Test eccentricityTransform.
    im = np.array([[2, 2, 2, 3, 3, 3, 3],
                   [2, 1, 1, 1, 1, 1, 3],
                   [2, 1, 1, 1, 1, 1, 3],
                   [2, 1, 1, 1, 1, 1, 3],
                   [2, 2, 2, 3, 3, 3, 3]], dtype=np.float32)
    s2 = 1.41421354
    imd = np.array([[2, s2+1, s2+2, s2+3, s2+2, s2+1, 2],
                    [1, s2+1, s2, 1, s2, s2+1, 1],
                    [0, 2, 1, 0, 1, 2, 0],
                    [1, s2+1, s2, 1, s2, s2+1, 1],
                    [2, s2+1, s2+2, s2+3, s2+2, s2+1, 2]])
    res = vigra.filters.eccentricityTransform(im)
    checkAboutSame(res, imd)
    assert_raises(ValueError, vigra.filters.eccentricityTransform, im.astype(np.int32))
    assert_raises(ValueError, vigra.filters.eccentricityTransform, np.zeros((5, 6, 7, 8), dtype=np.float32))

    # Test eccentricityCenters.
    c = vigra.filters.eccentricityCenters(im)
    assert(c[1] == (2, 3) and c[2] == (2, 0) and c[3] == (2, 6))
    assert_raises(ValueError, vigra.filters.eccentricityCenters, im.astype(np.int32))
    assert_raises(ValueError, vigra.filters.eccentricityCenters, np.zeros((5, 6, 7, 8), dtype=np.float32))
    
    # Test eccentricityTransformWithCenters.
    res, c = vigra.filters.eccentricityTransformWithCenters(im)
    checkAboutSame(res, imd)
    assert(c[1] == (2, 3) and c[2] == (2, 0) and c[3] == (2, 6))
    assert_raises(ValueError, vigra.filters.eccentricityTransformWithCenters, im.astype(np.int32))
    assert_raises(ValueError, vigra.filters.eccentricityTransformWithCenters, np.zeros((5, 6, 7, 8), dtype=np.float32))

def test_gaussianGradient():
    im = np.array([[0, 0, 0, 0, 0],
                   [0, 1, 1, 0, 0],
                   [0, 1, 1, 1, 0],
                   [0, 0, 1, 1, 0],
                   [0, 0, 0, 0, 0]], dtype=np.float64)
    imd0 = np.array([[[ -8.01406117e-18 ,  8.02309608e-18],
                     [ -9.18662066e-18 ,  8.61150404e-02],
                     [ -7.83858551e-18 , -1.21313252e-01],
                     [ -2.95644367e-18 , -1.88750139e-01],
                     [ -5.94896474e-19 ,  0.00000000e+00]],
                    [[  8.61150404e-02 , -1.30104261e-17],
                     [  1.29233242e-01 ,  1.29233242e-01],
                     [  2.03991002e-01 , -1.01871715e-01],
                     [  2.15420146e-01 , -2.15420146e-01],
                     [  1.88750139e-01 ,  1.73472348e-18]],
                    [[ -1.21313252e-01 , -1.73472348e-18],
                     [ -1.01871715e-01 ,  2.03991002e-01],
                     [  2.77555756e-17 , -2.58040117e-17],
                     [  1.01871715e-01 , -2.03991002e-01],
                     [  1.21313252e-01 , -2.94902991e-17]],
                    [[ -1.88750139e-01 , -1.38777878e-17],
                     [ -2.15420146e-01 ,  2.15420146e-01],
                     [ -2.03991002e-01 ,  1.01871715e-01],
                     [ -1.29233242e-01 , -1.29233242e-01],
                     [ -8.61150404e-02 ,  8.67361738e-19]],
                    [[ -5.94896474e-19 ,  0.00000000e+00],
                     [ -2.95644367e-18 ,  1.88750139e-01],
                     [ -7.83858551e-18 ,  1.21313252e-01],
                     [ -9.18662066e-18 , -8.61150404e-02],
                     [ -8.01406117e-18 ,  8.02309608e-18]]], dtype=np.float64)
    imd1 = np.array([[  1.13399844e-17 , 8.61150404e-02 , 1.21313252e-01 , 1.88750139e-01 , 5.94896474e-19],
                     [  8.61150404e-02 , 1.82763404e-01 , 2.28013542e-01 , 3.04650092e-01 , 1.88750139e-01],
                     [  1.21313252e-01 , 2.28013542e-01 , 3.78974801e-17 , 2.28013542e-01 , 1.21313252e-01],
                     [  1.88750139e-01 , 3.04650092e-01 , 2.28013542e-01 , 1.82763404e-01 , 8.61150404e-02],
                     [  5.94896474e-19 , 1.88750139e-01 , 1.21313252e-01 , 8.61150404e-02 , 1.13399844e-17]], dtype=np.float64)
    res = vigra.filters.gaussianGradient(im, 1)
    checkAboutSame(res, imd0)
    assert_raises(ValueError, vigra.filters.gaussianGradient, im)
    assert_raises(ValueError, vigra.filters.gaussianGradient, im.astype(np.uint8), 1)
    assert_raises(ValueError, vigra.filters.gaussianGradient, np.zeros((5, 6, 7, 8, 9), dtype=np.float64), 1)

    res = vigra.filters.gaussianGradientMagnitude(im, 1)
    checkAboutSame(res, imd1)
    assert_raises(ValueError, vigra.filters.gaussianGradientMagnitude, im)
    assert_raises(ValueError, vigra.filters.gaussianGradientMagnitude, im.astype(np.uint8), 1)
    assert_raises(ValueError, vigra.filters.gaussianGradientMagnitude, np.zeros((5, 6, 7, 8, 9, 10), dtype=np.float64), 1)

def test_hessianOfGaussian():
    im = np.array([[0, 0, 0, 0, 0],
                   [0, 1, 1, 0, 0],
                   [0, 1, 1, 1, 0],
                   [0, 0, 1, 1, 0],
                   [0, 0, 0, 0, 0]], dtype=np.float64)
    imd = np.array([[[  2.03117070e-01,  4.84491752e-34,  2.03117070e-01],
                     [  2.65611971e-01, -1.09103007e-18, -9.96424231e-02],
                     [  3.34984252e-01,  3.98564636e-18, -2.12308959e-01],
                     [  2.93927033e-01,  4.46302120e-18,  9.19749106e-02],
                     [  2.36835873e-01, -2.40741243e-35,  2.36835873e-01]],
                    [[ -9.96424231e-02,  3.46944695e-18,  2.65611971e-01],
                     [ -8.92026671e-02,  7.56238735e-02, -8.92026671e-02],
                     [ -1.41639200e-02,  5.51365585e-02, -2.62273105e-01],
                     [  7.17062698e-02, -2.89764871e-02,  7.17062698e-02],
                     [  9.19749106e-02,  7.80625564e-18,  2.93927033e-01]],
                    [[ -2.12308959e-01,  0.00000000e+00,  3.34984252e-01],
                     [ -2.62273105e-01,  5.51365585e-02, -1.41639200e-02],
                     [ -3.06656412e-01,  1.30341282e-01, -3.06656412e-01],
                     [ -2.62273105e-01,  5.51365585e-02, -1.41639200e-02],
                     [ -2.12308959e-01, -2.60208521e-18,  3.34984252e-01]],
                    [[  9.19749106e-02, -7.58941521e-18,  2.93927033e-01],
                     [  7.17062698e-02, -2.89764871e-02,  7.17062698e-02],
                     [ -1.41639200e-02,  5.51365585e-02, -2.62273105e-01],
                     [ -8.92026671e-02,  7.56238735e-02, -8.92026671e-02],
                     [ -9.96424231e-02,  0.00000000e+00,  2.65611971e-01]],
                    [[  2.36835873e-01, -2.40741243e-35,  2.36835873e-01],
                     [  2.93927033e-01, -4.46302120e-18,  9.19749106e-02],
                     [  3.34984252e-01, -3.98564636e-18, -2.12308959e-01],
                     [  2.65611971e-01,  1.09103007e-18, -9.96424231e-02],
                     [  2.03117070e-01,  4.84491752e-34,  2.03117070e-01]]], dtype=np.float64)
    res = vigra.filters.hessianOfGaussian(im, 1)
    checkAboutSame(res, imd)
    assert_raises(ValueError, vigra.filters.hessianOfGaussian, im)
    assert_raises(ValueError, vigra.filters.hessianOfGaussian, im.astype(np.uint8), 1)
    assert_raises(ValueError, vigra.filters.hessianOfGaussian, np.zeros((5, 6, 7, 8, 9), dtype=np.float32), 1)

def test_structureTensor():
    im = np.array([[0, 0, 0],
                   [0, 1, 1]], dtype=np.float64)
    imd = np.array([[[0, 0, 0.00163116],
                     [0, 0, 0.01856249],
                     [0, 0, 0.00163116]],
                    [[0, 0, 0.01856249],
                     [0, 0, 0.21124014],
                     [0, 0, 0.01856249]]], dtype=np.float64)
    res = vigra.filters.structureTensor(im, 0.2, 0.4)
    checkAboutSame(imd, res)
    assert_raises(ValueError, vigra.filters.structureTensor, im)
    assert_raises(ValueError, vigra.filters.structureTensor, im, 0.2)
    assert_raises(ValueError, vigra.filters.structureTensor, im.astype(np.uint8), 0.2, 0.4)
    assert_raises(ValueError, vigra.filters.structureTensor, np.zeros((5, 6, 7, 8, 9, 10), dtype=np.float64), 0.5, 0.75)

def test_vectorToTensor():
    im = np.zeros((2, 3, 2), dtype=np.float32)
    im[0, 1] = [4, 5]
    im[1, 1] = [3, 7]
    imd = np.array([[[0, 0, 0],
                     [16, 20, 25],
                     [0, 0, 0]],
                    [[0, 0, 0],
                     [9, 21, 49],
                     [0, 0, 0]]], dtype=np.float32)
    res = vigra.filters.vectorToTensor(im)
    checkAboutSame(res, imd)
    vigra.filters.vectorToTensor(np.zeros((3, 4, 5, 3), dtype=np.float32))
    assert_raises(ValueError, vigra.filters.vectorToTensor, im.astype(np.uint32))
    assert_raises(ValueError, vigra.filters.vectorToTensor, np.zeros((2, 3, 4), dtype=np.float32))
    assert_raises(ValueError, vigra.filters.vectorToTensor, np.zeros((3, 4, 5, 4), dtype=np.float32))

def test_tensorTraceDetEigen():
    # Test tensorTrace.
    im = np.array([[0, 0, 0],
                   [0, 1, 1]], dtype=np.float64)
    im = vigra.filters.structureTensor(im, 0.2, 0.4)
    imd = np.array([[ 0.00163116,  0.01856249,  0.00163116],
                    [ 0.01856249,  0.21124014,  0.01856249]], dtype=np.float64)
    res = vigra.filters.tensorTrace(im)
    checkAboutSame(res, imd)
    assert_raises(ValueError, vigra.filters.tensorTrace, im.astype(np.uint32))
    assert_raises(ValueError, vigra.filters.tensorTrace, np.zeros((3, 4, 5), dtype=np.float64))

    # Test tensorDeterminant.
    vigra.filters.tensorDeterminant(im)
    assert_raises(ValueError, vigra.filters.tensorDeterminant, im.astype(np.uint32))
    assert_raises(ValueError, vigra.filters.tensorDeterminant, np.zeros((3, 4, 5), dtype=np.float64))

    # Test tensorEigenvalues.
    imd = np.array([[[ 0.00163116, 0 ],
                     [ 0.01856249, 0 ],
                     [ 0.00163116, 0 ]],
                    [[ 0.01856249, 0 ],
                     [ 0.21124014, 0 ],
                     [ 0.01856249, 0 ]]], dtype=np.float64)
    res = vigra.filters.tensorEigenvalues(im)
    checkAboutSame(res, imd)
    assert_raises(ValueError, vigra.filters.tensorEigenvalues, im.astype(np.uint32))
    assert_raises(ValueError, vigra.filters.tensorEigenvalues, np.zeros((3, 4, 5), dtype=np.float64))

def test_applyColortable():
    vigra.colors.applyColortable(np.zeros((10, 11), dtype=np.uint8), np.zeros((10, 4), dtype=np.uint8))
    assert_raises(ValueError, vigra.colors.applyColortable, np.zeros((10, 11), dtype=np.uint8))
    assert_raises(ValueError, vigra.colors.applyColortable, np.zeros((10, 11), dtype=np.uint8), np.zeros((10, 4), dtype=np.uint32))
    assert_raises(ValueError, vigra.colors.applyColortable, np.zeros((10, 11, 12), dtype=np.uint8), np.zeros((10, 4), dtype=np.uint8))

def test_linearRangeMapping():
    im = np.zeros((10, 11), dtype=np.uint8)
    im[5, 5] = 128
    vigra.colors.linearRangeMapping(im)
    im = np.zeros((10, 11, 12, 13), dtype=np.uint8)
    im[5, 5, 5, 6] = 128
    assert_raises(ValueError, vigra.colors.linearRangeMapping, im)

def test_labelImage():
    im = np.array([[1, 1, 2, 2],
                   [1, 0, 2, 2],
                   [0, 0, 1, 1]], dtype=np.uint8)
    imd0 = np.array([[1, 1, 3, 3],
                    [1, 2, 3, 3],
                    [2, 2, 4, 4]], dtype=np.uint8)
    imd1 = np.array([[1, 1, 2, 2],
                     [1, 0, 2, 2],
                     [0, 0, 3, 3]], dtype=np.uint8)
    res = vigra.analysis.labelImage(im)
    checkEqual(res, imd0)
    vigra.analysis.labelImage(im, "direct")
    assert_raises(ValueError, vigra.analysis.labelImage, im.astype(np.float64))
    assert_raises(ValueError, vigra.analysis.labelImage, np.zeros((3, 4, 5), dtype=np.uint8))
    res = vigra.analysis.labelImageWithBackground(im)
    checkEqual(res, imd1)
    vigra.analysis.labelImageWithBackground(im, "direct")
    assert_raises(ValueError, vigra.analysis.labelImageWithBackground, im.astype(np.float64))
    assert_raises(ValueError, vigra.analysis.labelImageWithBackground, np.zeros((3, 4, 5), dtype=np.uint8))

def test_labelVolume():
    im = np.zeros((4, 5, 6), dtype=np.uint8)
    vigra.analysis.labelVolume(im)
    vigra.analysis.labelVolume(im, "direct")
    assert_raises(ValueError, vigra.analysis.labelVolume, im.astype(np.float64))
    assert_raises(ValueError, vigra.analysis.labelVolume, np.zeros((3, 4, 5, 6), dtype=np.uint8))
    vigra.analysis.labelVolumeWithBackground(im)
    vigra.analysis.labelVolumeWithBackground(im, "direct")
    assert_raises(ValueError, vigra.analysis.labelVolumeWithBackground, im.astype(np.float64))
    assert_raises(ValueError, vigra.analysis.labelVolumeWithBackground, np.zeros((3, 4, 5, 6), dtype=np.uint8))

def test_labelMultiArray():
    im = np.zeros((4, 5, 6, 7), dtype=np.uint8)
    vigra.analysis.labelMultiArray(im)
    vigra.analysis.labelMultiArray(im, "direct")
    assert_raises(ValueError, vigra.analysis.labelMultiArray, im, "direct", 0)
    assert_raises(ValueError, vigra.analysis.labelMultiArray, im.astype(np.float64))
    vigra.analysis.labelMultiArrayWithBackground(im)
    vigra.analysis.labelMultiArrayWithBackground(im, "direct")
    vigra.analysis.labelMultiArrayWithBackground(im, "direct", 0)
    assert_raises(ValueError, vigra.analysis.labelMultiArrayWithBackground, im, "direct", 0, 1)
    assert_raises(ValueError, vigra.analysis.labelMultiArrayWithBackground, im.astype(np.float64))

def test_extendedLocalMinima():
    im = np.array([[5, 4, 5, 6],
                   [6, 3, 4, 5],
                   [7, 4, 5, 6]], dtype=np.uint8)
    imd = np.array([[0, 0, 0, 0],
                    [0, 1, 0, 0],
                    [0, 0, 0, 0]], dtype=np.uint8)
    res = vigra.analysis.extendedLocalMinima(im, 1)
    checkEqual(res, imd)
    vigra.analysis.extendedLocalMinima(im, 1, 8)
    vigra.analysis.extendedLocalMinima(im.astype(np.float32))
    vigra.analysis.extendedLocalMinima(im.astype(np.float32), 1.5)
    assert_raises(ValueError, vigra.analysis.extendedLocalMinima, im)
    assert_raises(ValueError, vigra.analysis.extendedLocalMinima, im, 1, 8, 1)
    assert_raises(ValueError, vigra.analysis.extendedLocalMinima, im, 1.0)

def test_extendedLocalMinima3D():
    im = np.zeros((3, 4, 5), dtype=np.uint8)
    vigra.analysis.extendedLocalMinima3D(im)
    vigra.analysis.extendedLocalMinima3D(im, 5)
    vigra.analysis.extendedLocalMinima3D(im, 5, 6)
    assert_raises(ValueError, vigra.analysis.extendedLocalMinima3D, im, 1.5)
    assert_raises(ValueError, vigra.analysis.extendedLocalMinima3D, im, 5, 6, 1)

def test_watersheds():
    im_f = vigra.arraytypes.ScalarImage(np.random.rand(100, 200)*255, dtype=np.float32)
    im_i = vigra.arraytypes.ScalarImage(np.random.rand(100, 200)*255, dtype=np.uint32)

    vigra.analysis.watersheds(im_f)
    vigra.analysis.watersheds(im_f, seeds=im_i)
    vigra.analysis.watersheds(im_f, method="RegionGrowing")
    vigra.analysis.watersheds(im_f, seeds=im_i, method="RegionGrowing")
    assert_raises(ValueError, vigra.analysis.watersheds, im_f, seeds=im_i.astype(np.float32), method="RegionGrowing")
    assert_raises(ValueError, vigra.analysis.watersheds, im_f, seeds=np.zeros((100, 200, 3), dtype=np.uint32), method="RegionGrowing")
    assert_raises(ValueError, vigra.analysis.watersheds, np.zeros((100, 200, 3), dtype=np.float32), seeds=np.zeros((100, 200), dtype=np.uint32), method="RegionGrowing")

    vigra.analysis.watershedsNew(im_f)
    vigra.analysis.watershedsNew(im_f, seeds=im_i)
    vigra.analysis.watershedsNew(im_f, method="RegionGrowing")
    vigra.analysis.watershedsNew(im_f, seeds=im_i, method="RegionGrowing")
    assert_raises(ValueError, vigra.analysis.watershedsNew, im_f, seeds=im_i.astype(np.float32), method="RegionGrowing")
    assert_raises(ValueError, vigra.analysis.watershedsNew, im_f, seeds=np.zeros((100, 200, 3), dtype=np.uint32), method="RegionGrowing")
    assert_raises(ValueError, vigra.analysis.watershedsNew, np.zeros((100, 200, 3), dtype=np.float32), seeds=np.zeros((100, 200), dtype=np.uint32), method="RegionGrowing")

def test_superpixels():
    im_f = vigra.arraytypes.ScalarImage(np.random.rand(100, 200)*255, dtype=np.float32)
    im_i = vigra.arraytypes.ScalarImage(np.random.rand(100, 200)*255, dtype=np.uint32)

    vigra.analysis.slicSuperpixels(im_f, 0.5, 1)
    assert_raises(ValueError, vigra.analysis.slicSuperpixels, im_f)
    assert_raises(ValueError, vigra.analysis.slicSuperpixels, im_f, 0.5)
    assert_raises(ValueError, vigra.analysis.slicSuperpixels, np.zeros((100, 200, 10, 20), dtype=np.float32), 0.5, 1)



#######################################################################
#                                                                      
#         Copyright 2009-2010 by Ullrich Koethe                        
#                                                                      
#    This file is part of the VIGRA computer vision library.           
#    The VIGRA Website is                                              
#        http://hci.iwr.uni-heidelberg.de/vigra/                       
#    Please direct questions, bug reports, and contributions to        
#        ullrich.koethe@iwr.uni-heidelberg.de    or                    
#        vigra@informatik.uni-hamburg.de                               
#                                                                      
#    Permission is hereby granted, free of charge, to any person       
#    obtaining a copy of this software and associated documentation    
#    files (the "Software"), to deal in the Software without           
#    restriction, including without limitation the rights to use,      
#    copy, modify, merge, publish, distribute, sublicense, and/or      
#    sell copies of the Software, and to permit persons to whom the    
#    Software is furnished to do so, subject to the following          
#    conditions:                                                       
#                                                                      
#    The above copyright notice and this permission notice shall be    
#    included in all copies or substantial portions of the             
#    Software.                                                         
#                                                                      
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND    
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES   
#    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND          
#    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT       
#    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,      
#    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING      
#    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR     
#    OTHER DEALINGS IN THE SOFTWARE.                                   
#                                                                      
#######################################################################

from __future__ import division, print_function
import sys
print("\nexecuting test file", __file__, file=sys.stderr)
exec(compile(open('set_paths.py', "rb").read(), 'set_paths.py', 'exec'))

from nose.tools import assert_equal, raises, assert_raises
import vigra
import numpy as np
from vigra.analysis import *
from vigra.filters import *
import vigra.arraytypes as at

img_rgb_f = at.RGBImage(np.random.rand(100,200,3)*255,dtype=np.float32)
img_scalar_f = at.ScalarImage(np.random.rand(100,200)*255,dtype=np.float32)
img_multi_f = at.Vector3Image(np.random.rand(100,200,3)*255,dtype=np.float32)
 
img_rgb_i = at.RGBImage(np.random.rand(100,200,3)*255,dtype=np.int32)
img_scalar_i = at.ScalarImage(np.random.rand(100,200)*255,dtype=np.uint32)
img_scalar_i64 = at.ScalarImage(np.random.rand(199,199)*4095,dtype=np.uint64)
img_scalar_ui8 = at.ScalarImage(np.random.rand(100,200)*255,dtype=np.uint8)
img_multi_i = at.Vector3Image(np.random.rand(100,200,3)*255,dtype=np.int32)

vol_rgb_f = at.RGBVolume(np.random.rand(100,200,60,3)*255,dtype=np.float32)
vol_scalar_f = at.ScalarVolume(np.random.rand(100,200,50)*255,dtype=np.float32)
vol_multi_f = at.Vector6Volume(np.random.rand(100,200,50,6)*255,dtype=np.float32)
 
vol_rgb_i = at.RGBVolume(np.random.rand(100,200,60,3)*255,dtype=np.int32)
vol_scalar_i = at.ScalarVolume(np.random.rand(100,200,50)*255,dtype=np.int32)
vol_multi_i = at.Vector6Volume(np.random.rand(100,200,50,6)*255,dtype=np.int32)

def checkShape(shape1, shape2):
    if isinstance(shape1, np.ndarray):
        shape1 = shape1.shape
    if isinstance(shape2, np.ndarray):
        shape2 = shape2.shape
    assert_equal(shape1, shape2)

def checkType(i,type):
    assert_equal(i.dtype, type)

def checkAboutSame(i1,i2):
    compare_shapes(i1.shape, i2.shape)
    difference=np.sum(np.abs(i1-i2))/float(np.size(i1))
    assert(difference<5)
    
def test_watersheds():

    res = watershedsUnionFind(img_scalar_f)
    checkShape(img_scalar_f,res[0])
    checkType(res[0], np.uint32)

    res = watershedsUnionFind(img_scalar_f, 4)
    checkShape(img_scalar_f, res[0])
    checkType(res[0], np.uint32)

    img_scalar_i = at.ScalarImage(img_scalar_f.shape, dtype=np.uint32)
    res = watershedsUnionFind(img_scalar_f, 8, img_scalar_i)
    assert(res[0] is img_scalar_i)

    assert_raises(RuntimeError, watershedsUnionFind, img_scalar_f, 5, img_scalar_i)
    
    res = watersheds(img_scalar_f, seeds=img_scalar_i, method="RegionGrowing")
    checkShape(img_scalar_f, res[0])
    checkType(res[0], np.uint32)

    # 3D
    res = watershedsUnionFind(vol_scalar_f, 6)
    checkShape(vol_scalar_f, res[0])

    vol_scalar_i = at.ScalarVolume(vol_scalar_f.shape, dtype=np.uint32)
    res = watershedsUnionFind(vol_scalar_f, 26, vol_scalar_i)
    assert(res[0] is vol_scalar_i)
    
    assert_raises(RuntimeError, watershedsUnionFind, img_scalar_f, 17, img_scalar_i)

def test_MinimaMaxima():
    #2D Case
    res = localMinima(img_scalar_f)
    checkShape(img_scalar_f,res)
    checkType(res,img_scalar_f.dtype)
    
    res = extendedLocalMinima(img_scalar_f)
    checkShape(img_scalar_f,res)
    checkType(res,img_scalar_f.dtype)
    
    res = localMaxima(img_scalar_f)
    checkShape(img_scalar_f,res)
    checkType(res,img_scalar_f.dtype)
    
    res = extendedLocalMaxima(img_scalar_f)
    checkShape(img_scalar_f,res)
    checkType(res,img_scalar_f.dtype)
    
    res = labelImage(img_scalar_f)
    checkShape(img_scalar_f,res)
    checkType(res,np.uint32)
    
    res = labelImageWithBackground(img_scalar_f)
    checkShape(img_scalar_f,res)
    checkType(res,np.uint32)
    
    #3D Case
    res = localMinima3D(vol_scalar_f)
    checkShape(vol_scalar_f,res)
    checkType(res,vol_scalar_f.dtype)
    
    res = extendedLocalMinima3D(vol_scalar_f)
    checkShape(vol_scalar_f,res)
    checkType(res,vol_scalar_f.dtype)
    
    res = localMaxima3D(vol_scalar_f)
    checkShape(vol_scalar_f,res)
    checkType(res,vol_scalar_f.dtype)
    
    res = extendedLocalMaxima3D(vol_scalar_f)
    checkShape(vol_scalar_f,res)
    checkType(res,vol_scalar_f.dtype)
    
    data = vigra.taggedView(np.zeros((100,200,50),dtype=np.float32), 
                            vigra.defaultAxistags(3, noChannels=True))

    data[10,5,10]=1
    data[98,10,10]=1
    data[90,50,30]=1
    data[10,10,2]=1
    data[10,80,8]=1
    data[10,10,8]=1
    data[10,150,2]=1
    data[80,10,10]=1
    data[70,120,40]=1
    
    res = localMaxima3D(data,neighborhood=26)
    np.testing.assert_array_equal(res, data, "Error in the calculation of the Maxima")

    res = extendedLocalMaxima3D(data,neighborhood=26)
    np.testing.assert_array_equal(res, data, "Error in the calculation of the Extended Maxima")
    
    res = localMinima3D(1-data,neighborhood=26)
    np.testing.assert_array_equal(res, data, "Error in the calculation of the Minima")
    
    res = extendedLocalMinima3D(1-data,neighborhood=26)
    np.testing.assert_array_equal(res, data, "Error in the calculation of the Extended Minima")

def test_Region2Crack():
    res = regionImageToCrackEdgeImage(img_scalar_i)
    assert_equal(img_scalar_f.shape[0]*2-1, res.shape[0])
    assert_equal(img_scalar_f.shape[1]*2-1, res.shape[1])
    checkType(res,res.dtype)

    regionImageToCrackEdgeImage(img_scalar_i64[0:100,0:100], 1, img_scalar_i64)
    checkType(img_scalar_i64, np.uint64)
    
    res = regionImageToEdgeImage(img_scalar_i)
    checkShape(res,img_scalar_i)
    
def test_transforms():
    res = distanceTransform2D(img_scalar_f)
    checkShape(img_scalar_i, res)
    checkType(res, np.float32)
    
    res = distanceTransform2D(img_scalar_ui8)
    checkShape(img_scalar_ui8, res)
    checkType(res, np.float32)
    
    res = radialSymmetryTransform2D(img_scalar_f,1)
    checkShape(img_scalar_f, res)
    checkType(res, np.float32)

def test_cornerss():
    res = cornernessHarris(img_scalar_f,1)
    checkShape(img_scalar_f, res)
    checkType(res, np.float32)
    
    res = cornernessFoerstner(img_scalar_f,2)
    checkShape(img_scalar_f, res)
    checkType(res, np.float32)
    
    res = cornernessRohr(img_scalar_f,0.5)
    checkShape(img_scalar_f, res)
    checkType(res, np.float32)
    
    res = cornernessBeaudet(img_scalar_f,1)
    checkShape(img_scalar_f, res)
    checkType(res, np.float32)

def test_edges():    
    res = cannyEdgeImage(img_scalar_f, 1,128,255)
    checkShape(img_scalar_f, res)
    checkType(res, np.uint8)
    
    res = cannyEdgeImageWithThinning(img_scalar_f, 1,128,255)
    checkShape(img_scalar_f, res)
    checkType(res, np.uint8)
    
    res = shenCastanEdgeImage(img_scalar_f, 1,128,255)
    checkShape(img_scalar_f, res)
    checkType(res, np.uint8)
    
    res = shenCastanCrackEdgeImage(img_scalar_f, 1,128,255)
    assert_equal(img_scalar_f.shape[0]*2-1, res.shape[0])
    assert_equal(img_scalar_f.shape[1]*2-1, res.shape[1])
    
    res1 = beautifyCrackEdgeImage(res,  1, 0)
    checkShape(res1, res)
    checkType(res1, np.uint8)
    
    res1 = closeGapsInCrackEdgeImage(res, 4)
    checkShape(res1, res)
    checkType(res1, np.uint8)
    
    res = removeShortEdges(img_scalar_ui8, 10, 0)
    checkShape(img_scalar_ui8, res)
    checkType(res, np.uint8)
    
    res = boundaryTensor2D(img_scalar_f, 1)
    assert_equal(img_scalar_f.shape[0], res.shape[0])
    assert_equal(img_scalar_f.shape[1], res.shape[1])
    assert_equal(res.shape[2], 3)    
    checkType(res, np.float32)
    
    res = hourGlassFilter2D(img_multi_f, 1, 2)
    assert_equal(img_multi_f.shape[0], res.shape[0])
    assert_equal(img_multi_f.shape[1], res.shape[1])
    checkType(res, np.float32)
    
    res = tensorEigenRepresentation2D(img_multi_f)
    assert_equal(img_multi_f.shape[0], res.shape[0])
    assert_equal(img_multi_f.shape[1], res.shape[1])
    checkType(res, np.float32)
    
    res = tensorTrace(img_multi_f)
    assert_equal(img_multi_f.shape[0], res.shape[0])
    assert_equal(img_multi_f.shape[1], res.shape[1])
    checkType(res, np.float32)
    
    res = rieszTransformOfLOG2D(img_scalar_f, 1, 1, 1)
    assert_equal(img_multi_f.shape[0], res.shape[0])
    assert_equal(img_multi_f.shape[1], res.shape[1])
    checkType(res, np.float32)
    
    
def ok_():
    print(".", file=sys.stderr)
        



import vigra
import numpy
import matplotlib.pyplot as plt

f = "islands.png"
cmap = plt.get_cmap("afmhot")

# Compute labeled iamge.
img = numpy.squeeze(vigra.readImage(f))
lbl = numpy.array(vigra.analysis.labelImage(img))

# Compute the eccentricity transform.
ecc = vigra.filters.eccentricityTransform(lbl)
plt.imshow(numpy.swapaxes(ecc, 1, 0), cmap=cmap)
plt.show()

# Compute the eccentricity centers and draw them into the image.
centers = vigra.filters.eccentricityCenters(lbl)
m = ecc.max()
for c in centers[1:]:
    ecc[c] = m
plt.imshow(numpy.swapaxes(ecc, 1, 0), cmap=cmap)
plt.show()

# # Compute the transformation and the centers in one step:
# ecc, centers = vigra.filters.eccentricityTransformWithCenters(lbl)



from __future__ import division, print_function
import sys
print("\nexecuting test file", __file__, file=sys.stderr)
exec(compile(open('set_paths.py', "rb").read(), 'set_paths.py', 'exec'))

import numpy
import vigra

def _impl_test_labelMultiArray(dtype):
    a = numpy.zeros( (10,10,10,10,2), dtype=dtype )
    a[3:4, 5:8, 5:8, 5:8] = 100
    a[4:5, 8:10, 5:8, 5:8] = 100 # touches above object on corners only.
    
    a[3:4, 1:4, 5:8, 5:8] = 200
    
    labeled = vigra.analysis.labelMultiArray(a)
    assert labeled.dtype == numpy.uint32
    assert labeled.max() == 4

    labeled = vigra.analysis.labelMultiArrayWithBackground(a)
    assert labeled.dtype == numpy.uint32
    assert labeled.max() == 3

    labeled = vigra.analysis.labelMultiArrayWithBackground(a, neighborhood='indirect')
    assert labeled.dtype == numpy.uint32
    assert labeled.max() == 2
    
    labeled = vigra.analysis.labelMultiArrayWithBackground(a, background_value=100)
    assert labeled.dtype == numpy.uint32
    assert labeled.max() == 2

def test_labelMultiArray():
    _impl_test_labelMultiArray(numpy.uint8)
    _impl_test_labelMultiArray(numpy.uint32)
    _impl_test_labelMultiArray(numpy.float32)


def _impl_test_applyMapping(dtype):
    original = numpy.arange(100, dtype=dtype ).reshape(10,10)
    mapping = dict( zip( original.flat[:], original.flat[:] + 100 ) )

    # Not in-place
    remapped = vigra.analysis.applyMapping(original, mapping)
    assert remapped.dtype == original.dtype, "Default output dtype did not match input dtype!"
    assert (remapped == original+100).all()

    # in-place
    original_copy = original.copy()
    vigra.analysis.applyMapping(original_copy, mapping, out=original_copy)
    assert (original_copy == original+100).all()

    # Different dtypes
    mapping = dict( zip( original.flat[:], (original.flat[:] + 100).astype(numpy.uint64) ) )

    result = numpy.zeros_like( original, dtype=numpy.uint64 )
    vigra.analysis.applyMapping(original, mapping, out=result)
    assert (result == original+100).all()

    mapping = dict( zip( original.flat[:], (original.flat[:] + 100).astype(numpy.uint8) ) )

    result = numpy.zeros_like( original, dtype=numpy.uint8 )
    vigra.analysis.applyMapping(original, mapping, out=result)
    assert (result == original+100).all()

    # Incomplete mapping
    for i in range(10):
        del mapping[i]

    remapped = vigra.analysis.applyMapping(original, mapping, allow_incomplete_mapping=True)
    assert (remapped[0] == original[0]).all()
    assert (remapped[1:] == original[1:]+100).all()
    
    try:
        remapped = vigra.analysis.applyMapping(original, mapping, allow_incomplete_mapping=False)
    except KeyError:
        pass
    else:
        assert False, "Expected to get an exception due to the incomplete mapping!"

def test_applyMapping():
    _impl_test_applyMapping(numpy.uint8)
    _impl_test_applyMapping(numpy.uint32)
    _impl_test_applyMapping(numpy.uint64)

def _impl_test_unique(dtype):
    a = numpy.array([2,3,5,7,11,13,17,19,23,29] + [2,3,5,7,11,13,17,19,23,29], dtype=dtype)
    u = vigra.analysis.unique(a, sort=True)
    assert (u == [2,3,5,7,11,13,17,19,23,29]).all()

def test_unique():
    _impl_test_unique(numpy.uint8)
    _impl_test_unique(numpy.uint32)
    _impl_test_unique(numpy.uint64)

def _impl_relabelConsecutive(dtype):
    start = 17
    a = numpy.random.randint(start,start+100, size=(100,100) ).astype(dtype)
    a[-1] = numpy.arange(start,start+100, dtype=dtype) # Make sure every number is used
    a[:] *= 3
    consecutive, maxlabel, mapping = vigra.analysis.relabelConsecutive(a, start)

    assert consecutive.dtype == a.dtype, "Default output dtype did not match input dtype!"
    assert maxlabel == consecutive.max()
    assert (vigra.analysis.applyMapping(a, mapping) == consecutive).all()

    assert (numpy.unique(consecutive) == numpy.arange(start,start+100)).all(), \
        "relabeled array does not have consecutive labels"
    
    first_label_a = a[0,0]
    first_label_c = consecutive[0,0]
    assert ((a == first_label_a) == (consecutive == first_label_c)).all()

    # Now in-place
    orig = a.copy()
    consecutive, maxlabel, mapping = vigra.analysis.relabelConsecutive(a, start, out=a)
    assert consecutive is a
    assert maxlabel == consecutive.max()
    assert (vigra.analysis.applyMapping(orig, mapping) == consecutive).all()
    
    assert (numpy.unique(a) == numpy.arange(start,start+100)).all(), \
        "relabeled array does not have consecutive labels"
    
    first_label_orig = orig[0,0]
    first_label_a = a[0,0]
    assert ((orig == first_label_orig) == (a == first_label_a)).all()

def test_relabelConsecutive():
    _impl_relabelConsecutive(numpy.uint8)
    _impl_relabelConsecutive(numpy.uint32)
    _impl_relabelConsecutive(numpy.uint64)

def test_relabelConsecutive_keep_zeros():
    a = numpy.arange(1,10, dtype=numpy.uint8)
    a[1::2] = 0 # replace even numbers with zeros

    # Check keep_zeros=True
    consecutive, maxlabel, mapping = vigra.analysis.relabelConsecutive(a, start_label=100, keep_zeros=True)
    assert (consecutive[1::2] == 0).all(), \
        "Zeros were not left untouched!"
    assert set(consecutive[0::2]) == set(range(100,100+len(consecutive[0::2]))), \
        "Non-zero items were not correctly consecutivized!"
    assert maxlabel == consecutive.max()
    assert (vigra.analysis.applyMapping(a, mapping) == consecutive).all()

    # Check keep_zeros=False
    consecutive, maxlabel, mapping = vigra.analysis.relabelConsecutive(a, start_label=100, keep_zeros=False)
    assert (numpy.unique(consecutive) == numpy.arange(100, 100+1+len(a[::2]))).all(), \
        "relabeled array does not have consecutive labels: {}".format(consecutive)
    assert maxlabel == consecutive.max()
    assert (vigra.analysis.applyMapping(a, mapping) == consecutive).all()

def ok_():
    print(".", file=sys.stderr)



def hook(ui, repo, **args):
    import os
    repoPath = repo.url()[5:] + '/' # cut-off 'file:' prefix
    testSuccessFile = repoPath + 'test/testSuccess'
    if not os.path.exists(testSuccessFile):
        print("File 'test/testSuccess' is missing. Run the test suite before committing.")
        return True
    testTime = os.path.getmtime(testSuccessFile)
    stat = repo.status()
    files = stat[0]+stat[1]  # added or modified files
    modified = []
    for file in files:
        if file[-4:] not in ['.cxx', '.hxx']:
            continue
        fileTime =  os.path.getmtime(repoPath+file)
        if fileTime > testTime:
            modified.append(file)
    if len(modified) > 0:
        print("Run the test suite before committing. The following files are untested:")
        for file in modified:
            print('   ',file)
        return True
    return False
    



import vigra
from vigra import graphs

# parameter:
filepath = '12003.jpg'   # input image path
sigmaGradMag = 2.0       # sigma Gaussian gradient
superpixelDiameter = 10  # super-pixel size
slicWeight = 10.0        # SLIC color - spatial weight
gamma = 0.15             # exp(-gamma * edgeIndicator)
edgeThreshold = 2.5      # values higher are considered as edges
scale = 1.0              # how much smoothing
iterations = 10          # how man smoothing iterations

# load image and convert to LAB
img = vigra.impex.readImage(filepath)

res = vigra.filters.nonlinearDiffusion(img, 1.9, 20.0)

vigra.imshow(res)
vigra.show()


from __future__ import division, print_function
import sys
print("\nexecuting test file", __file__, file=sys.stderr)
exec(compile(open('set_paths.py', "rb").read(), 'set_paths.py', 'exec'))

import vigra
import numpy as np


gaus1=10*np.random.randn(1000).reshape((500,2))+np.array([20,20])
gaus1=gaus1.astype(np.float32)
gaus2=10*np.random.randn(1000).reshape((500,2))+np.array([20,20])
gaus2=gaus2.astype(np.float32)
label_gaus1=np.ones(500).astype(np.uint32)
label_gaus2=np.zeros(500).astype(np.uint32)

def test_rf_learn():
    RF=vigra.learning.RandomForest(10)
    fmat=np.vstack([gaus1,gaus2])
    lmat=np.vstack([label_gaus1,label_gaus2]).reshape(-1,1)
    RF.learnRF(fmat,lmat,0,1,100)

def ok_():
    print(".", file=sys.stderr)


import vigra
import vigra.graphs as graphs

# parameter:
filepath = '12003.jpg'   # input image path
sigmaGradMag = 2.0       # sigma Gaussian gradient
superpixelDiameter = 10  # super-pixel size
slicWeight = 10.0        # SLIC color - spatial weight
k = 10                   # free parameter in felzenszwalbs method
nodeNumStop = 500        # desired num. nodes in result

# load image and convert to LAB
img = vigra.impex.readImage(filepath)

# get super-pixels with slic on LAB image
imgLab = vigra.colors.transform_RGB2Lab(img)
labels, nseg = vigra.analysis.slicSuperpixels(imgLab, slicWeight,
                                              superpixelDiameter)
labels = vigra.analysis.labelImage(labels)

# compute gradient on interpolated image
imgLabBig = vigra.resize(imgLab, [imgLab.shape[0]*2-1, imgLab.shape[1]*2-1])
gradMag = vigra.filters.gaussianGradientMagnitude(imgLabBig, sigmaGradMag)

# get 2D grid graph and  edgeMap for grid graph
# from gradMag of interpolated image
gridGraph = graphs.gridGraph(img.shape[0:2])
gridGraphEdgeIndicator = graphs.edgeFeaturesFromInterpolatedImage(gridGraph,
                                                                  gradMag)

# get region adjacency graph from super-pixel labels
rag = graphs.regionAdjacencyGraph(gridGraph, labels)

# accumulate edge weights from gradient magnitude
edgeWeights = rag.accumulateEdgeFeatures(gridGraphEdgeIndicator)

# do the segmentation with felzenszwalbs method
labels   = graphs.felzenszwalbSegmentation(rag, edgeWeights, 
                                           k=50,nodeNumStop=nodeNumStop)


rag.show(img, labels)
vigra.show()



import vigra
from vigra import graphs
from vigra import numpy
import pylab

# parameter
filepath = '12003.jpg'  # input image path
sigmaGradMag = 5.0      # sigma Gaussian gradient
superpixelDiameter = 10 # super-pixel size
slicWeight = 10.0       # SLIC color - spatial weight
beta = 0.5              # node vs edge weight
nodeNumStop = 50        # desired num. nodes in result


# load image and convert to LAB
img = vigra.impex.readImage(filepath)

# get super-pixels with slic on LAB image
imgLab = vigra.colors.transform_RGB2Lab(img)
labels, nseg = vigra.analysis.slicSuperpixels(imgLab, slicWeight,
                                              superpixelDiameter)
labels = vigra.analysis.labelImage(labels)

# compute gradient on interpolated image
imgLabBig = vigra.resize(imgLab, [imgLab.shape[0]*2-1, imgLab.shape[1]*2-1])
gradMag = vigra.filters.gaussianGradientMagnitude(imgLabBig, sigmaGradMag)

# get 2D grid graph and  edgeMap for grid graph
# from gradMag of interpolated image
gridGraph = graphs.gridGraph(img.shape[0:2])
gridGraphEdgeIndicator = graphs.edgeFeaturesFromInterpolatedImage(gridGraph,
                                                                  gradMag)
# get region adjacency graph from super-pixel labels
rag = graphs.regionAdjacencyGraph(gridGraph, labels)

# accumulate edge weights from gradient magnitude
edgeWeights = rag.accumulateEdgeFeatures(gridGraphEdgeIndicator)

# accumulate node features from grid graph node map
# which is just a plain image (with channels)
nodeFeatures = rag.accumulateNodeFeatures(imgLab)

# do agglomerativeClustering
labels = graphs.agglomerativeClustering(graph=rag, edgeWeights=edgeWeights,
                                        beta=beta, nodeFeatures=nodeFeatures,
                                        nodeNumStop=nodeNumStop,wardness=0.8)

# show result
f = pylab.figure()
ax1 = f.add_subplot(2, 2, 1)
vigra.imshow(gradMag,show=False)
ax1.set_title("Input Image")
pylab.axis('off')

ax2 = f.add_subplot(2, 2, 2)
rag.show(img)
ax2.set_title("Over-Segmentation")
pylab.axis('off')

ax3 = f.add_subplot(2, 2, 3)
rag.show(img, labels)
ax3.set_title("Result-Segmentation")
pylab.axis('off')

ax4 = f.add_subplot(2, 2, 4)
rag.showNested(img, labels)
ax4.set_title("Result-Segmentation")
pylab.axis('off')

vigra.show()



#######################################################################
#                                                                      
#         Copyright 2009-2013 by Ullrich Koethe and Thorben Kroeger
#                                                                      
#    This file is part of the VIGRA computer vision library.           
#    The VIGRA Website is                                              
#        http://hci.iwr.uni-heidelberg.de/vigra/                       
#    Please direct questions, bug reports, and contributions to        
#        ullrich.koethe@iwr.uni-heidelberg.de    or                    
#        vigra@informatik.uni-hamburg.de                               
#                                                                      
#    Permission is hereby granted, free of charge, to any person       
#    obtaining a copy of this software and associated documentation    
#    files (the "Software"), to deal in the Software without           
#    restriction, including without limitation the rights to use,      
#    copy, modify, merge, publish, distribute, sublicense, and/or      
#    sell copies of the Software, and to permit persons to whom the    
#    Software is furnished to do so, subject to the following          
#    conditions:                                                       
#                                                                      
#    The above copyright notice and this permission notice shall be    
#    included in all copies or substantial portions of the             
#    Software.                                                         
#                                                                      
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND    
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES   
#    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND          
#    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT       
#    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,      
#    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING      
#    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR     
#    OTHER DEALINGS IN THE SOFTWARE.                                   
#                                                                      
#######################################################################

from __future__ import division, print_function
import sys
print("\nexecuting test file", __file__, file=sys.stderr)
exec(compile(open('set_paths.py', "rb").read(), 'set_paths.py', 'exec'))

from nose.tools import assert_equal, raises, assert_raises, assert_true
import vigra
import numpy

_have_qt = False
try:
    from PyQt4.QtGui import QImage
    from qimage2ndarray import byte_view
    _have_qt = True
except:
    pass

def test_gray2qimage():
    if not _have_qt:
        return
    
    # create test data such that rounding does not depend on the least significant bit
    a = (numpy.random.randint(255, size=(100,200)).astype(numpy.float32)+0.25)/255.0-0.5
    a[0,0] =-0.5 #make sure we get the correct bounds
    a[0,1] = 0.5
    vigra.impex.writeImage(a.swapaxes(0,1), "tmp1.png")

    img = QImage(a.shape[1], a.shape[0], QImage.Format_ARGB32_Premultiplied)
    n = numpy.asarray([-0.5, 0.5], dtype=numpy.float32)
    vigra.colors.gray2qimage_ARGB32Premultiplied(a, byte_view(img), n)
    img.save("tmp2.png")
    
    tmp1 = vigra.impex.readImage("tmp1.png")
    tmp2 = vigra.impex.readImage("tmp2.png")
    
    for i in range(3):
        assert_true( (tmp1 == tmp2[:,:,i]).all() )
    assert_true( (tmp2[:,:,3] == 255).all() )

def test_alphamodulated2qimage():
    if not _have_qt:
        return
    
    # create test data such that rounding does not depend on the least significant bit
    a = (numpy.random.randint(255, size=(100,200)).astype(numpy.float32)+0.25)/255.0-0.5
    a[0,0] =-0.5 #make sure we get the correct bounds
    a[0,1] = 0.5
    vigra.impex.writeImage(a.swapaxes(0,1), "tmp1.png")
    
    tintColor = numpy.asarray([0.0, 1.0, 0.0], dtype=numpy.float32)
    
    img = QImage(a.shape[1], a.shape[0], QImage.Format_ARGB32_Premultiplied)
    n = numpy.asarray([-0.5, 0.5], dtype=numpy.float32)
    
    vigra.colors.alphamodulated2qimage_ARGB32Premultiplied(a, byte_view(img), tintColor, n)
    
    tmp1 = vigra.impex.readImage("tmp1.png").view(numpy.ndarray).squeeze().astype(numpy.uint8)
    tmp2 = byte_view(img)
    tmp2 = tmp2.swapaxes(0,1)
    
    assert_true( tmp1.shape[0:2] == tmp2.shape[0:2] )
    assert_true( tmp2.ndim == 3 )
    assert_true( tmp2.shape[2] == 4 )
    
    assert_true( (tmp2[:,:,0] == 0).all() )
    assert_true( (tmp2[:,:,2] == 0).all() )
    assert_true( (tmp2[:,:,3] == tmp1).all() )
    assert_true( (tmp2[:,:,1] == tmp1).all() )

if __name__ == '__main__':
    import nose
    nose.run(defaultTest=__file__)



from __future__ import print_function
import vigra
from vigra import graphs
from vigra import numpy
import pylab
# parameter
filepath = '12003.jpg'  # input image path
sigmaGradMag = 2.0      # sigma Gaussian gradient
superpixelDiameter = 10 # super-pixel size
slicWeight = 10.0       # SLIC color - spatial weight
beta = 0.5              # node vs edge weight
nodeNumStop = 50        # desired num. nodes in result


# load image and convert to LAB
img = vigra.impex.readImage(filepath)

# get super-pixels with slic on LAB image
imgLab = vigra.colors.transform_RGB2Lab(img)
labels, nseg = vigra.analysis.slicSuperpixels(imgLab, slicWeight,
                                              superpixelDiameter)
labels = vigra.analysis.labelImage(labels)


gridGraph = graphs.gridGraph(img.shape[0:2])
rag = graphs.regionAdjacencyGraph(gridGraph, labels)

# get the merge graph
mg = graphs.mergeGraph(rag)

# do n runs where we erase k edges in each run
n = 3
k = 200
for r in range(n):

    erased = 0

    while(erased<k):

        # get a random edge
        randEdgeId = numpy.random.randint(rag.edgeNum)

        print("random edge:",randEdgeId)
        # edge could be gone
        # -since we could have merged it already
        # - or due to transitivity of other merges
        if mg.hasEdgeId(randEdgeId):
            mg.contractEdge(mg.edgeFromId(randEdgeId))
            erased+=1

    # get the segmentation
    # (brand new c++ function for max speed)
    labels = mg.graphLabels()

    # view results
    rag.show(img,labels)
    vigra.show()

    # get the result as pixels wise labeling
    asImage = rag.projectLabelsToGridGraph(labels)
    asImage = vigra.taggedView(asImage, "xy")



#######################################################################
#
#         Copyright 2009-2010 by Ullrich Koethe
#
#    This file is part of the VIGRA computer vision library.
#    The VIGRA Website is
#        http://hci.iwr.uni-heidelberg.de/vigra/
#    Please direct questions, bug reports, and contributions to
#        ullrich.koethe@iwr.uni-heidelberg.de    or
#        vigra@informatik.uni-hamburg.de
#
#    Permission is hereby granted, free of charge, to any person
#    obtaining a copy of this software and associated documentation
#    files (the "Software"), to deal in the Software without
#    restriction, including without limitation the rights to use,
#    copy, modify, merge, publish, distribute, sublicense, and/or
#    sell copies of the Software, and to permit persons to whom the
#    Software is furnished to do so, subject to the following
#    conditions:
#
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the
#    Software.
#
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
#    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#    OTHER DEALINGS IN THE SOFTWARE.
#
#######################################################################

import numpy
import copy

vigraTypecastingRules = '''
Default output types are thus determined according to the following rules:

   1. The output type does not depend on the order of the arguments::

         a + b results in the same type as b + a

   2.a With exception of logical functions and abs(), the output type
       does not depend on the function to be executed.

   2.b The output type of logical functions is bool.

   2.c The output type of abs() follows general rules unless the
       input contains complex numbers, in which case the output type
       is the corresponding float number type::

         a + b results in the same type as a / b
         a == b => bool
         abs(complex128) => float64

   3. If the inputs have the same type, the type is preserved::

         uint8 + uint8 => uint8

   4. If (and only if) one of the inputs has at least 64 bits, the output
      will also have at least 64 bits::

         int64 + uint32 => int64
         int64 + 1.0    => float64

   5. If an array is combined with a scalar of the same kind (integer,
      float, or complex), the array type is preserved. If an integer
      array with at most 32 bits is combined with a float scalar, the
      result is float32 (and rule 4 kicks in if the array has 64 bits)::

         uint8   + 1   => uint8
         uint8   + 1.0 => float32
         float32 + 1.0 => float32
         float64 + 1.0 => float64

   6. Integer expressions with mixed types always produce signed results.
      If the arguments have at most 32 bits, the result will be int32,
      otherwise it will be int64 (cf. rule 4)::

         int8  + uint8  => int32
         int32 + uint8  => int32
         int32 + uint32 => int32
         int32 + int64  => int64
         int64 + uint64 => int64

   7. In all other cases, the output type is equal to the highest input
      type::

         int32   + float32    => float32
         float32 + complex128 => complex128

   8. All defaults can be overridden by providing an explicit output array::

         ufunc.add(uint8, uint8, uint16) => uint16

In order to prevent overflow, necessary upcasting is performed before
the function is executed.
'''

class Function(object):
    test_types = numpy.typecodes['AllInteger'][:-2] + numpy.typecodes['AllFloat']+'O'
    len_test_types = len(test_types)
    kindToNumber = {'b': 1, 'u': 2, 'i': 2, 'f': 3, 'c': 4}
    boolFunctions = ['equal', 'greater', 'greater_equal', 'less', 'less_equal', 'not_equal',
                     'logical_and', 'logical_not', 'logical_or', 'logical_xor']

    def __init__(self, function):
        self.function = function
        self.is_bool = function.__name__ in self.boolFunctions
        self.is_abs  = function.__name__ == "absolute"
        self.__doc__ = function.__doc__
        self.nin = function.nin
        self.nout = function.nout

    def __getattr__(self, name):
        return getattr(self.function, name)

    def __repr__(self):
        return "<vigra.ufunc '%s'>" % self.__name__

    def priorities(self, *args):
        '''Among the inputs with largest size, find the one with highest
           __array_priority__. Return this input, or None if there is no
           inputs with 'size' and '__array_priority__' defined.'''
        maxSize = max([getattr(x, 'size', 0) for x in args])
        if maxSize == 0:
            return None
        priorities = [(getattr(x, '__array_priority__', -1.0), x) for x in args if getattr(x, 'size', 0) == maxSize]
        priorities = sorted(priorities,key = lambda tuplepx: tuplepx[0])
        if priorities[-1][0] == -1.0:
            return None
        else:
            return priorities[-1][1]

    def common_type_numpy(self, *args):
        '''Find a common type for the given inputs.
           This function will become obsolete when numpy.find_common_type() will be fixed.
           Code taken from the partial fix in numpy changeset 7133.
        '''
        arrayTypes = [x.dtype for x in args if hasattr(x, 'dtype')]
        N = len(arrayTypes)
        if N == 1:
            highestArrayType = arrayTypes[0]
        else:
            k = 0
            while k < self.len_test_types:
                highestArrayType = numpy.dtype(self.test_types[k])
                numcoerce = len([x for x in arrayTypes if highestArrayType >= x])
                if numcoerce == N:
                    break
                k += 1
        scalarTypes = [numpy.dtype(type(x)) for x in args if numpy.isscalar(x)]
        N = len(scalarTypes)
        if N == 0 or highestArrayType >= scalarTypes[0]:
            return (highestArrayType, highestArrayType)
        else:
            h, s = highestArrayType.kind, scalarTypes[0].kind
            if (h in ['i', 'u'] and s in ['i', 'u']) or \
               (h == 'f' and s == 'f') or \
               (h == 'c' and s == 'c'):
                return (highestArrayType, highestArrayType)
            return (highestArrayType, scalarTypes[0])

    def common_type(self, *args):
        '''Find the appropriate pair (in_dtype, out_dtype) according to
           vigranumpy typecasting rules. in_dtype is the type into which
           the arguments will be casted before performing the operation
           (to prevent possible overflow), out_type is the type the output
           array will have (unless an explicit out-argument is provided).

           See ufunc.vigraTypecastingRules for detailed information on coercion rules.
        '''
        if self.is_abs and args[0].dtype.kind == "c" and args[1] is None:
            dtype = args[0].dtype
            if dtype == numpy.complex64:
                return dtype, numpy.float32
            if dtype == numpy.complex128:
                return dtype, numpy.float64
            if dtype == numpy.clongdouble:
                return dtype, numpy.longdouble
        arrayTypes = [(self.kindToNumber[x.dtype.kind], x.dtype.itemsize, x.dtype) for x in args if hasattr(x, 'dtype')]
        arrayTypes.sort()
        if arrayTypes[0] != arrayTypes[-1] and arrayTypes[-1][0] == 2:
            if arrayTypes[-1][1] <= 4:
                highestArrayType = (2, 4, numpy.int32)
            else:
                highestArrayType = (2, 8, numpy.int64)
        else:
            highestArrayType = arrayTypes[-1]

        if self.is_bool:
            return (highestArrayType[-1], numpy.bool8)

        scalarType = [numpy.dtype(type(x)) for x in args if numpy.isscalar(x)]
        if not scalarType:
            return (highestArrayType[-1], highestArrayType[-1])
        scalarType = (self.kindToNumber[scalarType[0].kind], scalarType[0].itemsize, scalarType[0])
        if highestArrayType[0] >= scalarType[0]:
            return (highestArrayType[-1], highestArrayType[-1])
        elif scalarType[0] == 3 and highestArrayType[1] <= 4:
            return (highestArrayType[-1], numpy.float32)
        else:
            return (highestArrayType[-1], scalarType[-1])

class UnaryFunction(Function):
    def __call__(self, arg, out=None):
        a = arg.squeeze().transposeToNumpyOrder()
        dtype, out_dtype = self.common_type(a, out)

        if out is None:
            out = arg.__class__(arg, dtype=out_dtype, order='A', init=False)
            o = out.squeeze().transposeToNumpyOrder()
        else:
            o = out.squeeze().transposeToNumpyOrder()
            if not a.axistags.compatible(o.axistags):
                raise RuntimeError("%s(): axistag mismatch" % self.function.__name__)

        a = numpy.require(a, dtype).view(numpy.ndarray) # view(ndarray) prevents infinite recursion
        self.function(a, o)
        return out

class UnaryFunctionOut2(Function):
    def __call__(self, arg, out1=None, out2=None):
        a = arg.squeeze().transposeToNumpyOrder()
        dtype, out_dtype = self.common_type(a, out1, out2)

        if out1 is None:
            out1 = arg.__class__(arg, dtype=out_dtype, order='A', init=False)
            o1 = out1.squeeze().transposeToNumpyOrder()
        else:
            o1 = out1.squeeze().transposeToNumpyOrder()
            if not a.axistags.compatible(o1.axistags):
                raise RuntimeError("%s(): axistag mismatch" % self.function.__name__)

        if out2 is None:
            out2 = arg.__class__(arg, dtype=out_dtype, order='A', init=False)
            o2 = out2.squeeze().transposeToNumpyOrder()
        else:
            o2 = out2.squeeze().transposeToNumpyOrder()
            if not a.axistags.compatible(o2.axistags):
                raise RuntimeError("%s(): axistag mismatch" % self.function.__name__)

        a = numpy.require(a, dtype).view(numpy.ndarray) # view(ndarray) prevents infinite recursion
        self.function(a, o1, o2)
        return out1, out2

class BinaryFunction(Function):
    def __call__(self, arg1, arg2, out=None):
        if arg1.__class__ is numpy.ndarray or arg2.__class__ is numpy.ndarray:
            return self.function(arg1, arg2, out)

        dtype, out_dtype = self.common_type(arg1, arg2, out)

        if isinstance(arg1, numpy.ndarray):
            a1 = arg1.transposeToNumpyOrder()
            if isinstance(arg2, numpy.ndarray):
                a2 = arg2.transposeToNumpyOrder()

                if arg1.__array_priority__ == arg2.__array_priority__:
                    priorityArg = arg2 if arg1.ndim < arg2.ndim else arg1
                else:
                    priorityArg = arg2 if arg1.__array_priority__ < arg2.__array_priority__ else arg1

                if a1.ndim < a2.ndim:
                    a1 = a1.insertChannelAxis(order='C')
                elif a1.ndim > a2.ndim:
                    a2 = a2.insertChannelAxis(order='C')

                axistags = a1.axistags

                if not axistags.compatible(a2.axistags):
                    raise RuntimeError("%s(): input axistag mismatch %r vs. %r" %
                                         (self.function.__name__, axistags, a2.axistags))
                shape = tuple(max(k) for k in zip(a1.shape, a2.shape))
                a2 = numpy.require(a2, dtype).view(numpy.ndarray)
            else:
                priorityArg = arg1
                axistags = a1.axistags
                shape = a1.shape
                a2 = arg2
            a1 = numpy.require(a1, dtype).view(numpy.ndarray) # view(ndarray) prevents infinite recursion
        else:
            a1 = arg1
            a2 = arg2.transposeToNumpyOrder()
            axistags = a2.axistags
            shape = a2.shape
            priorityArg = arg2
            a2 = numpy.require(a2, dtype).view(numpy.ndarray)

        if out is None:
            outClass = priorityArg.__class__
            inversePermutation = priorityArg.permutationFromNumpyOrder()
            o = outClass(shape, dtype=out_dtype, order='C', axistags=axistags, init=False)
            if priorityArg.ndim < o.ndim:
                out = o.dropChannelAxis().transpose(inversePermutation)
            else:
                out = o.transpose(inversePermutation)
        else:
            o = out.transposeToNumpyOrder()
            if o.ndim < len(shape):
                o = o.insertChannelAxis(order='C')
            if not axistags.compatible(o.axistags):
                raise RuntimeError("%s(): output axistag mismatch %r vs. %r" %
                                         (self.function.__name__, axistags, o.axistags))
        self.function(a1, a2, o)
        return out

__all__ = []

for _k in numpy.__dict__.values():
     if type(_k) == numpy.ufunc:
        if _k.nin == 1 and _k.nout == 1:
            exec(_k.__name__ + " = UnaryFunction(_k)")
        if _k.nin == 1 and _k.nout == 2:
            exec(_k.__name__ + " = UnaryFunctionOut2(_k)")
        if _k.nin == 2:
            exec(_k.__name__ + " = BinaryFunction(_k)")
        __all__.append(_k.__name__)

__all__ = sorted(__all__)

def _prepareDoc():
    doc = '''
The following mathematical functions are available in this module
(refer to numpy for detailed documentation)::

'''

    k = 0
    while k < len(__all__):
        t = 8
        while True:
            d = '    ' + '   '.join(__all__[k:k+t]) + '\n'
            if len(d) <= 80:
                break
            t -= 1
        doc += d
        k += t

    return doc + '''
Some of these functions are also provided as member functions of
VigraArray::

    __abs__   __add__   __and__   __div__   __divmod__   __eq__
    __floordiv__   __ge__   __gt__   __invert__   __le__   __lshift__
    __lt__   __mod__   __mul__   __ne__   __neg__   __or__   __pos__
    __pow__   __radd__   __radd__   __rand__   __rdiv__   __rdivmod__
    __rfloordiv__   __rlshift__   __rmod__   __rmul__   __ror__   __rpow__
    __rrshift__   __rshift__   __rsub__   __rtruediv__   __rxor__   __sub__
    __truediv__   __xor__

As usual, these functions are applied independently at each pixel.

Vigranumpy overloads the numpy-versions of these functions in order to make their
behavior more suitable for image analysis. In particular, we changed two aspects:

* Axistag consistency is checked, and the order of axes and strides is
  preserved in the result array. (In contrast, plain numpy functions
  always create C-order arrays, disregarding the stride order of the
  inputs.)
* Typecasting rules are changed such that (i) data are represented with
  at most 32 bits, when possible, (ii) the number of types that occur as
  results of mixed expressions is reduced, and (iii) the chance of bad
  surprises is minimized.

''' + vigraTypecastingRules

__doc__ = _prepareDoc()


#######################################################################
#                                                                      
#         Copyright 2009-2010 by Ullrich Koethe                        
#                                                                      
#    This file is part of the VIGRA computer vision library.           
#    The VIGRA Website is                                              
#        http://hci.iwr.uni-heidelberg.de/vigra/                       
#    Please direct questions, bug reports, and contributions to        
#        ullrich.koethe@iwr.uni-heidelberg.de    or                    
#        vigra@informatik.uni-hamburg.de                               
#                                                                      
#    Permission is hereby granted, free of charge, to any person       
#    obtaining a copy of this software and associated documentation    
#    files (the "Software"), to deal in the Software without           
#    restriction, including without limitation the rights to use,      
#    copy, modify, merge, publish, distribute, sublicense, and/or      
#    sell copies of the Software, and to permit persons to whom the    
#    Software is furnished to do so, subject to the following          
#    conditions:                                                       
#                                                                      
#    The above copyright notice and this permission notice shall be    
#    included in all copies or substantial portions of the             
#    Software.                                                         
#                                                                      
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND    
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES   
#    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND          
#    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT       
#    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,      
#    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING      
#    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR     
#    OTHER DEALINGS IN THE SOFTWARE.                                   
#                                                                      
#######################################################################

from __future__ import division, print_function
import sys
print("\nexecuting test file", __file__, file=sys.stderr)
exec(compile(open('set_paths.py', "rb").read(), 'set_paths.py', 'exec'))

from nose.tools import assert_equal, raises
import numpy as np
from vigra import *
from vigra import arraytypes as at
from vigra.filters import *
from vigra.analysis import *

img_rgb_f = at.RGBImage(np.random.rand(100,200,3)*255,dtype=np.float32)
img_scalar_f = at.ScalarImage(np.random.rand(100,200)*255,dtype=np.float32)
img_multi_f = at.Vector4Image(np.random.rand(100,200,4)*255,dtype=np.float32)
img_3_f = at.Vector3Image(np.random.rand(100,200,3)*255,dtype=np.float32)
 
img_rgb_i = at.RGBImage(np.random.rand(100,200,3)*255,dtype=np.int32)
img_scalar_i = at.ScalarImage(np.random.rand(100,200)*255,dtype=np.int32)
img_multi_i = at.Vector4Image(np.random.rand(100,200,4)*255,dtype=np.int32)

vol_rgb_f = at.RGBVolume(np.random.rand(100,200,60,3)*255,dtype=np.float32)
vol_scalar_f = at.ScalarVolume(np.random.rand(100,200,50)*255,dtype=np.float32)
vol_multi_f = at.Vector6Volume(np.random.rand(100,200,50,6)*255,dtype=np.float32)
 
vol_rgb_i = at.RGBVolume(np.random.rand(100,200,60,3)*255,dtype=np.int32)
vol_scalar_i = at.ScalarVolume(np.random.rand(100,200,50)*255,dtype=np.int32)
vol_multi_i = at.Vector6Volume(np.random.rand(100,200,50,6)*255,dtype=np.int32)

def checkShape(shape1, shape2):
    if isinstance(shape1, np.ndarray):
        shape1 = shape1.shape
    if isinstance(shape2, np.ndarray):
        shape2 = shape2.shape
    assert_equal(shape1, shape2)

def checkImages(i1,i2):
    checkShape(i1.shape, i2.shape)
    assert(np.sum(i1==i2)!=0)

def checkAboutSame(i1,i2):
    checkShape(i1.shape, i2.shape)
    difference=np.sum(np.abs(i1-i2))/float(np.size(i1))
    assert(difference<5)
    
def test_watersheds():
    res = watersheds(img_scalar_f)
    checkShape(res[0].shape, img_scalar_f.shape)
    
def test_structureTensor():
    res = structureTensor(img_scalar_f,1.0,2.0, out=img_3_f)
    res = structureTensor(img_scalar_f,1.0,2.0)
    res = structureTensor(img_rgb_f,1.0,2.0)
    checkShape(res.shape, img_scalar_f.shape[:2] + (3,))
    
def test_simpleSharpening():
    res = simpleSharpening2D(img_scalar_f)
    
def test_gaussianSharpening():
    res = gaussianSharpening2D(img_scalar_f)
    
def test_convolution():
    krnl = gaussianKernel(0.5)
    
    k2_ = Kernel2D()
    k2_.initDisk(10)
    
    #k3 = gaussianDerivativeKernel(sigma, order)
    #guassianDerivative(img, sx,ox,sy,oy,sz,oz)
    #guassianDerivative(img, (sx,sy, sz), (ox, oy,oz))
    
    k2=Kernel2D()
    k2.initExplicitly((-1,-1),(1,1),np.array([[0,1,2],[1,2,3],[2,3,4]],dtype=np.float64))
    res = convolve(img_scalar_f, k2)


def test_multiconvolution():
   vol = vol_scalar_f
   k=Kernel1D()
   k.initGaussian(0.5)
   a=convolveOneDimension(vol, 0, k)
   a=convolveOneDimension(a, 1, k)
   a=convolveOneDimension(a, 2, k)
         
   b=convolve(vol, k)
     
   c=convolve(vol, (k, k, k))
   checkAboutSame(a, b)
   checkAboutSame(a, c)
    



import vigra
import vigra.graphs as graphs
import pylab


# parameter:
filepath = '100075.jpg'   # input image path
sigmaGradMag = 3.0       # sigma Gaussian gradient
superpixelDiameter = 10  # super-pixel size
slicWeight = 10.0        # SLIC color - spatial weight

# load image and convert to LAB
img = vigra.impex.readImage(filepath)

# get super-pixels with slic on LAB image
imgLab = vigra.colors.transform_RGB2Lab(img)
labels, nseg = vigra.analysis.slicSuperpixels(imgLab, slicWeight,
                                              superpixelDiameter)
labels = vigra.analysis.labelImage(labels)

# compute gradient
imgLabBig = vigra.resize(imgLab, [imgLab.shape[0]*2-1, imgLab.shape[1]*2-1])
gradMag    = vigra.filters.gaussianGradientMagnitude(imgLab, sigmaGradMag)
gradMagBig = vigra.filters.gaussianGradientMagnitude(imgLabBig, sigmaGradMag*2.0)

vigra.imshow(gradMagBig)
vigra.show()

# get 2D grid graph and  edgeMap for grid graph
# from gradMag of interpolated image
gridGraph = graphs.gridGraph(img.shape[0:2])
gridGraphEdgeIndicator = graphs.edgeFeaturesFromInterpolatedImage(gridGraph,
                                                                  gradMagBig)

# get region adjacency graph from super-pixel labels
rag = graphs.regionAdjacencyGraph(gridGraph, labels)


# accumulate edge  and ndie weights from gradient magnitude
ragEdgeWeights = rag.accumulateEdgeFeatures(gridGraphEdgeIndicator)
ragNodeWeights = rag.accumulateNodeFeatures(gradMag)

# generate seeds
seeds = graphs.nodeWeightedWatershedsSeeds(rag, ragNodeWeights)

# node weighted watersheds
labelsNodeWeighted  = graphs.nodeWeightedWatersheds(rag, ragNodeWeights, seeds)

# edge weighted watersheds
labelsEdgeWeighted  = graphs.edgeWeightedWatersheds(rag, ragEdgeWeights, seeds)


f = pylab.figure()
ax0 = f.add_subplot(1, 2, 1)
rag.showNested(img, labelsNodeWeighted)
ax0.set_title("node weighted")

ax1 = f.add_subplot(1, 2, 2)
rag.showNested(img, labelsEdgeWeighted)
ax1.set_title("edge weighted")
pylab.show()



#!/usr/bin/env python
from __future__ import division, print_function

import re
import glob
import sys
import xml.etree.ElementTree as ET
from itertools import chain

if len(sys.argv) != 2:
    print('usage: python makeFunctionIndex.py directory')
    sys.exit(1)

path = str(sys.argv[1])


def getClassList():
    tree = ET.parse("tagfile.tag")
    root = tree.getroot()

    class_list = []
    for classes in root.findall("*[@kind='class']"):
        long_name = classes.find('name').text
        # only classes from vigra namespace
        if long_name[0:5] == 'vigra':
            html = classes.find('filename').text
            short_name = long_name[long_name.rfind('::')+2:]
            namespace = long_name[:long_name.rfind('::')]
            class_list.append([html, short_name, namespace])

    class_list.sort(key = lambda a: a[1])
    return class_list



def getFunctionList():
    tree = ET.parse("tagfile.tag")
    root = tree.getroot()

    function_list = []
    for function in chain(root.findall("*[@kind='group']/*[@kind='function']"),root.findall("*[@kind='namespace']/*[@kind='function']")):
        name = function.find('name').text
        anchorfile = function.find('anchorfile').text
        anchor = function.find('anchor').text
        html = anchorfile + '#' + anchor
        function_list.append((html, name))

    # add special documentation for argument object factories
    for k in ['srcImageRange', 'srcImage', 'destImageRange', 'destImage', 'maskImage']:
        function_list.append(('group__ImageIterators.html#ImageBasedArgumentObjectFactories', k))
    for k in ['srcMultiArrayRange', 'srcMultiArray', 'destMultiArrayRange', 'destMultiArray']:
        function_list.append(('group__ImageIterators.html#MultiArrayBasedArgumentObjectFactories', k))
    for k in ['srcIterRange', 'srcIter', 'destIterRange', 'destIter', 'maskIter']:
        function_list.append(('group__ImageIterators.html#IteratorBasedArgumentObjectFactories', k))

    unique = set()
    set_add = unique.add
    function_list = [ x for x in function_list if x not in unique and not set_add(x)]
    function_list.sort(key = lambda a: a[1])
    function_list = disambiguateOverloadedFunctions(function_list)
    return function_list

def addHeading(index, initial):
    index = index + '<p><a name="index_' + initial + \
    '"><table class="function_index"><tr><th> ' + initial.upper() + \
    ' </th><td align="right" width="100%">VIGRA_NAVIGATOR_PLACEHOLDER</td></tr></table><p>\n'
    return index

def disambiguateOverloadedFunctions(functionList):
    for i in range(len(functionList)):
        overloaded = False
        functionName = functionList[i][1]
        if i > 0:
            lastFunctionName = functionList[i-1][1]
            if functionName == lastFunctionName:
                overloaded = True
        if i < len(functionList) - 1:
            nextFunctionName = functionList[i+1][1]
            if functionName == nextFunctionName:
                overloaded = True
        if overloaded:
            # disambiguate overloaded functions by their group or namespace
            link = functionList[i][0]
            group = re.sub(r'(group__|namespacevigra_1_1)([^\.]+)\.html.*', r'\2', link)
        else:
            group = ""
        functionList[i] = functionList[i] + (group,)

    return functionList


def generateFunctionIndex(functionList):
    index = ""
    initials = []
    for i in range(len(functionList)):
        functionName = functionList[i][1]
        link = functionList[i][0]
        initial = functionName[0]
        if i > 0:
            lastInitial = functionList[i-1][1][0]
            if initial != lastInitial:
                initials.append(initial)
                index = addHeading(index, initial)
        else:
            initials.append(initial)
            index = addHeading(index, initial)

        index = index + '<a href="'+ link + '">' + functionName + '</a>()'
        overloadDisambiguation = functionList[i][2]
        if overloadDisambiguation != "":
            index = index + ' [' + overloadDisambiguation + ']'
        index = index + '<br>\n'

    navigator = '['
    for i in range(len(initials)):
        initial = initials[i]
        if i != 0:
            navigator = navigator + '|'
        navigator = navigator + ' <a href="#index_' + initial + '">' + initial.upper() + '</a> '
    navigator = navigator + ']'
    index = re.sub('VIGRA_NAVIGATOR_PLACEHOLDER', navigator, index)

    # use file "/namespaces.html" as boiler plate for "/functionindex.html"
    text = open(path + "/namespaces.html").read()
    if text.find('</h1>') > -1: # up to doxygen 1.7.1
        header = text[:text.find('</h1>')+5]
    else: # for doxygen 1.7.4 to 1.7.6.1
        header = text[:re.search(r'<div class="title">[^<]*</div>\s*</div>\s*</div>(?:<!--header-->)?\n<div class="contents">',text).end()]
    footer = re.search(r'(?s)(<!-- footer.html -->.*)', text).group(1)

    text = re.sub(r'Namespace List', r'Function Index', header)
    text = text + '\n<p><hr>\n'
    text = text + index
    text = text + footer

    open(path + "/functionindex.html", 'w+').write(text)


classList = getClassList()
functionList = getFunctionList()
generateFunctionIndex(functionList)

# Export class and function list to c_api_replaces.txt for
# crosslinking of vigranumpy documentation.
# Note that '::' are not allowed in reStructuedText link names,
# so we have to use '.' instead.
try:
    replaces=open("../vigranumpy/docsrc/c_api_replaces.txt","w")
except IOError:
    print("Cannot open the 'c_api_replaces.txt' file, skipping.")
else:
    lowercase_names = set()
    for i in range(len(functionList)):
        functionName = functionList[i][1]
        overloadDisambiguation = functionList[i][2]
        if i > 0 and functionName == functionList[i-1][1] and \
                    overloadDisambiguation == functionList[i-1][2]:
            continue
        if overloadDisambiguation != "":
            functionName = overloadDisambiguation +'.' + functionName
        link = functionList[i][0]
        replaces.write(functionName+":"+link+"\n")
        lowercase_names.add(functionName.lower())
    for i in range(len(classList)):
        className = classList[i][1]
        namespace = classList[i][2]
        if (i > 0 and className == classList[i-1][1]) or \
        (i < len(classList)-1 and className == classList[i+1][1]):
            namespace = namespace.replace('::', '.')
            className = namespace +'.' + className
        if className.lower() in lowercase_names:
            className = className + 'Class'
        link = classList[i][0]
        replaces.write(className+":"+link+"\n")
    replaces.close()



from __future__ import print_function

import vigra
from vigra import graphs
from vigra import numpy
from vigra import Timer
from vigra import blockwise as bw


numpy.random.seed(42)

# input
shape = (500, 500, 500)

data = numpy.random.rand(*shape).astype('float32')

print("make options object")
options = bw.BlockwiseConvolutionOptions3D()
print(type(options))

sigma = 1.0
options.stdDev = (sigma, )*3
options.blockShape = (128, )*3

print("stddev",options.stdDev)
print("call blockwise filter")

with vigra.Timer("AllThread"):
    res = bw.gaussianSmooth(data, options)
with vigra.Timer("1thread"):
    resRef = vigra.gaussianSmoothing(data, sigma)


print(numpy.sum(numpy.abs(res-resRef)))



#######################################################################
#                                                                      
#         Copyright 2009-2010 by Ullrich Koethe                        
#                                                                      
#    This file is part of the VIGRA computer vision library.           
#    The VIGRA Website is                                              
#        http://hci.iwr.uni-heidelberg.de/vigra/                       
#    Please direct questions, bug reports, and contributions to        
#        ullrich.koethe@iwr.uni-heidelberg.de    or                    
#        vigra@informatik.uni-hamburg.de                               
#                                                                      
#    Permission is hereby granted, free of charge, to any person       
#    obtaining a copy of this software and associated documentation    
#    files (the "Software"), to deal in the Software without           
#    restriction, including without limitation the rights to use,      
#    copy, modify, merge, publish, distribute, sublicense, and/or      
#    sell copies of the Software, and to permit persons to whom the    
#    Software is furnished to do so, subject to the following          
#    conditions:                                                       
#                                                                      
#    The above copyright notice and this permission notice shall be    
#    included in all copies or substantial portions of the             
#    Software.                                                         
#                                                                      
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND    
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES   
#    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND          
#    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT       
#    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,      
#    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING      
#    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR     
#    OTHER DEALINGS IN THE SOFTWARE.                                   
#                                                                      
#######################################################################

from __future__ import division, print_function
import sys
print("\nexecuting test file", __file__, file=sys.stderr)
exec(compile(open('set_paths.py', "rb").read(), 'set_paths.py', 'exec'))


from nose.tools import assert_equal, raises
import numpy as np
import numpy as numpy
from vigra import graphs as vigraph
from vigra import graphs,taggedView
#from vigra.arraytypes import taggedView
import vigra

def testGridGraphSegmentationFelzenszwalbSegmentation():
    dataRGB  = numpy.random.random([3,3,3]).astype(numpy.float32)
    dataRGB  = taggedView(dataRGB,'xyc')
    data  = numpy.random.random([3,3]).astype(numpy.float32)
    edata = numpy.random.random([3*2-1,3*2-1]).astype(numpy.float32)
    g0 = graphs.gridGraph(data.shape)

    ew = graphs.edgeFeaturesFromInterpolatedImage(g0,edata)

    labels = graphs.felzenszwalbSegmentation(graph=g0,edgeWeights=ew,k=1.0,nodeNumStop=5)

    g1  = graphs.regionAdjacencyGraph(graph=g0,labels=labels)
    assert g1.nodeNum == 5


    
    data  = numpy.random.random([3,3,3]).astype(numpy.float32)
    edata = numpy.random.random([3*2-1,3*2-1,3*2-1]).astype(numpy.float32)
    g0 = graphs.gridGraph(data.shape)

    ew = graphs.edgeFeaturesFromInterpolatedImage(g0,edata)

    labels = graphs.felzenszwalbSegmentation(graph=g0,edgeWeights=ew,k=1.0,nodeNumStop=15)

    g1  = graphs.regionAdjacencyGraph(graph=g0,labels=labels)
    assert g1.nodeNum == 15

def testGridGraphWatersheds():

    data  = numpy.random.random([10,10,10]).astype(numpy.float32)
    edata = numpy.random.random([10*2-1,10*2-1,10*2-1]).astype(numpy.float32)
    g0 = graphs.gridGraph(data.shape)


    ew = graphs.edgeFeaturesFromInterpolatedImage(graph=g0,image=edata)

    # generate seeds
    seeds = graphs.nodeWeightedWatershedsSeeds(graph=g0,nodeWeights=data)
    # node weighted watershed seeds
    labelsNodeWeightedA  = graphs.nodeWeightedWatersheds(graph=g0,nodeWeights=data,seeds=seeds)
    # node weighted watershed seeds
    labelsNodeWeightedB  = graphs.nodeWeightedWatersheds(graph=g0,nodeWeights=data)
    # edge weighted watershed seeds
    seeds = graphs.nodeWeightedWatershedsSeeds(graph=g0,nodeWeights=data)
    labelsEdgeWeighted  = graphs.edgeWeightedWatersheds(graph=g0,edgeWeights=ew,seeds=seeds)

    assert numpy.array_equal(labelsNodeWeightedA,labelsNodeWeightedB)

    data  = numpy.random.random([10,10]).astype(numpy.float32)
    edata = numpy.random.random([10*2-1,10*2-1]).astype(numpy.float32)
    g0 = graphs.gridGraph(data.shape)


    ew = graphs.edgeFeaturesFromInterpolatedImage(graph=g0,image=edata)

    # generate seeds
    seeds = graphs.nodeWeightedWatershedsSeeds(graph=g0,nodeWeights=data)
    # node weighted watershed seeds
    labelsNodeWeightedA  = graphs.nodeWeightedWatersheds(graph=g0,nodeWeights=data,seeds=seeds)
    # node weighted watershed seeds
    labelsNodeWeightedB  = graphs.nodeWeightedWatersheds(graph=g0,nodeWeights=data)
    # edge weighted watershed seeds
    labelsEdgeWeighted  = graphs.edgeWeightedWatersheds(graph=g0,edgeWeights=ew,seeds=seeds)

    assert numpy.array_equal(labelsNodeWeightedA,labelsNodeWeightedB)


def testGridGraphAgglomerativeClustering():
    dataRGB  = numpy.random.random([10,10,3]).astype(numpy.float32)
    dataRGB  = vigra.taggedView(dataRGB,'xyc')
    data  = numpy.random.random([10,10]).astype(numpy.float32)
    edata = numpy.random.random([10*2-1,10*2-1]).astype(numpy.float32)
    g0 = graphs.gridGraph(data.shape)


    ew = graphs.edgeFeaturesFromInterpolatedImage(graph=g0,image=edata)
    #ew = taggedView(ew,'xyz')

    
    labels = graphs.agglomerativeClustering(graph=g0,edgeWeights=ew,nodeFeatures=dataRGB,nodeNumStop=5)
    g1  = graphs.regionAdjacencyGraph(graph=g0,labels=labels)
    assert g1.nodeNum == 5
    
    labels = graphs.agglomerativeClustering(graph=g0,edgeWeights=ew,nodeNumStop=5)
    g1  = graphs.regionAdjacencyGraph(graph=g0,labels=labels)
    assert g1.nodeNum == 5



    dataRGB  = numpy.random.random([10,10,10,3]).astype(numpy.float32)
    dataRGB  = vigra.taggedView(dataRGB,'xyzc')
    data  = numpy.random.random([10,10,10]).astype(numpy.float32)
    edata = numpy.random.random([10*2-1,10*2-1,10*2-1]).astype(numpy.float32)
    g0 = graphs.gridGraph(data.shape)


    ew = graphs.edgeFeaturesFromInterpolatedImage(graph=g0,image=edata)
    #ew = taggedView(ew,'xyz')

    
    labels = graphs.agglomerativeClustering(graph=g0,edgeWeights=ew,nodeFeatures=dataRGB,nodeNumStop=5)
    g1  = graphs.regionAdjacencyGraph(graph=g0,labels=labels)
    assert g1.nodeNum == 5
    
    labels = graphs.agglomerativeClustering(graph=g0,edgeWeights=ew,nodeNumStop=5)
    g1  = graphs.regionAdjacencyGraph(graph=g0,labels=labels)
    assert g1.nodeNum == 5

class TestGraph(object):

    def testAddNodesWithIds(self):

        IV = vigraph.INVALID

        g  = vigraph.listGraph()
        assert g.nodeNum == 0
        assert g.edgeNum == 0
        assert g.nodeFromId(0)==IV
        assert g.nodeFromId(1)==IV
        assert g.nodeFromId(2)==IV
        assert g.nodeFromId(3)==IV
        assert g.nodeFromId(4)==IV
        assert g.nodeFromId(5)==IV
        assert g.nodeFromId(6)==IV
        assert g.nodeFromId(7)==IV

        n5 =g.addNode(5)

        assert g.id(n5)==5

        assert n5!=IV
        assert g.nodeNum == 1
        assert g.edgeNum == 0
        assert g.nodeFromId(0)==IV
        assert g.nodeFromId(1)==IV
        assert g.nodeFromId(2)==IV
        assert g.nodeFromId(3)==IV
        assert g.nodeFromId(4)==IV
        assert g.nodeFromId(5)!=IV
        assert g.nodeFromId(6)==IV
        assert g.nodeFromId(7)==IV

        n2=g.addNode(2)
        assert n2!=IV
        assert g.nodeNum == 2
        assert g.nodeFromId(0)==IV
        assert g.nodeFromId(1)==IV
        assert g.nodeFromId(2)!=IV
        assert g.nodeFromId(3)==IV
        assert g.nodeFromId(4)==IV
        assert g.nodeFromId(5)!=IV
        assert g.nodeFromId(6)==IV
        assert g.nodeFromId(7)==IV

        n6=g.addNode(6)
        assert n6!=IV
        assert g.nodeNum == 3
        assert g.nodeFromId(0)==IV
        assert g.nodeFromId(1)==IV
        assert g.nodeFromId(2)!=IV
        assert g.nodeFromId(3)==IV
        assert g.nodeFromId(4)==IV
        assert g.nodeFromId(5)!=IV
        assert g.nodeFromId(6)!=IV
        assert g.nodeFromId(7)==IV

        n6=g.addNode(5)
        n2=g.addNode(2)
        n6=g.addNode(6)
        assert g.nodeNum == 3
        assert g.nodeFromId(0)==IV
        assert g.nodeFromId(1)==IV
        assert g.nodeFromId(2)!=IV
        assert g.nodeFromId(3)==IV
        assert g.nodeFromId(4)==IV
        assert g.nodeFromId(5)!=IV
        assert g.nodeFromId(6)!=IV
        assert g.nodeFromId(7)==IV

    def testAddEdges(self):
        IV = vigraph.INVALID
        elist = [
            [1,3],
            [1,5],
            [5,7],
            [3,4]
        ] 

        edges = np.array(elist,dtype=np.uint32)
        nodeIds = np.unique(edges.reshape(-1))


        g = vigraph.listGraph()

        edgeIds = g.addEdges(edges)

        assert g.edgeNum == len(elist)
        assert g.edgeNum == len(edgeIds)
        assert g.nodeNum == len(nodeIds)
        assert g.maxNodeId == nodeIds.max()

        for ui,vi in elist :
            assert g.findEdge(ui,vi)!=IV
            assert g.findEdge(g.nodeFromId(ui),g.nodeFromId(vi))!=IV

        for nId in nodeIds :
            nId = int(nId)
            assert g.nodeFromId(nId)!=IV

        for eId in edgeIds :
            eId = int(eId)
            assert g.edgeFromId(eId)!=IV

        findEdges = g.findEdges(edges)

        assert np.array_equal(findEdges,edgeIds)

    def testIters(self):
        g  = vigraph.listGraph()
        
        nodes = [n for n in g.nodeIter()]
        assert len(nodes)==0

        g.addNode(3)
        nodes = [n for n in g.nodeIter()]
        assert len(nodes)==1
        assert g.id(nodes[0]) == 3 

        g.addNode(6)
        nodes = [n for n in g.nodeIter()]
        assert len(nodes)==2
        assert g.id(nodes[0]) == 3 
        assert g.id(nodes[1]) == 6 

        g.addNode(2)
        nodes = [n for n in g.nodeIter()]
        assert len(nodes)==3
        assert g.id(nodes[0]) == 2 
        assert g.id(nodes[1]) == 3 
        assert g.id(nodes[2]) == 6 


import vigra
from vigra import graphs

filepath = '12003.jpg'
img = vigra.impex.readImage(filepath).astype('float32')[:,:,0]

res = vigra.filters.shockFilter(img,sigma=1.5, rho=10.0, updwindFactorH=1.0, iterations=5)
res = res.squeeze()

import numpy as np
import pylab
import matplotlib.cm as cm

f = pylab.figure()
for n, arr in enumerate([img,res]):
    arr= arr.squeeze().T
    #f.add_subplot(2, 1, n)  # this line outputs images on top of each other
    f.add_subplot(1, 2, n+1)  # this line outputs images side-by-side
    pylab.imshow(arr,cmap=cm.Greys_r)
pylab.title('( III x) image')
pylab.show()



import vigra
import vigra.graphs as graphs
import pylab
import numpy
import matplotlib

# parameter:
filepath = '100075.jpg'   # input image path
sigmaGradMag = 3.0       # sigma Gaussian gradient
superpixelDiameter = 100  # super-pixel size
slicWeight = 50.0        # SLIC color - spatial weight

# load image and convert to LAB
img = vigra.impex.readImage(filepath)

# get super-pixels with slic on LAB image
imgLab = vigra.colors.transform_RGB2Lab(img)
labels, nseg = vigra.analysis.slicSuperpixels(imgLab, slicWeight,
                                              superpixelDiameter)
labels = vigra.analysis.labelImage(labels)

# compute gradient
imgLabBig = vigra.resize(imgLab, [imgLab.shape[0]*2-1, imgLab.shape[1]*2-1])
gradMag    = vigra.filters.gaussianGradientMagnitude(imgLab, sigmaGradMag)
gradMagBig = vigra.filters.gaussianGradientMagnitude(imgLabBig, sigmaGradMag*2.0)



# get 2D grid graph and  edgeMap for grid graph
# from gradMag of interpolated image
gridGraph = graphs.gridGraph(img.shape[0:2])
gridGraphEdgeIndicator = graphs.edgeFeaturesFromInterpolatedImage(gridGraph,
                                                                  gradMagBig)

# get region adjacency graph from super-pixel labels
rag = graphs.regionAdjacencyGraph(gridGraph, labels)


cycles = graphs.find3CyclesEdges(rag)


for c in range(cycles.shape[0]):
    cic = cycles[c,:]


    f = numpy.zeros(rag.edgeNum)
    f[cic] = 1
    rag.showEdgeFeature(img, f)
    vigra.show()



from __future__ import print_function

import vigra
import vigra.graphs as vigraph
import pylab
import numpy
np=numpy
import sys
import matplotlib
import pylab as plt
import math
from matplotlib.widgets import Slider, Button, RadioButtons

def makeWeights(gamma):
    global hessian,gradmag,gridGraph
    print("hessian",hessian.min(),hessian.max())
    print("raw ",raw.min(),raw.max())
    wImg= numpy.exp((gradmag**0.5)*gamma*-1.0)#**0.5
    wImg = numpy.array(wImg).astype(numpy.float32)
    w=vigra.graphs.implicitMeanEdgeMap(gridGraph,wImg)
    return w

def makeVisuImage(path,img):
    coords = (path[:,0],path[:,1])
    visuimg =img.copy()
    iR=visuimg[:,:,0]
    iG=visuimg[:,:,1]
    iB=visuimg[:,:,2]
    iR[coords]=255
    iG[coords]=0
    iB[coords]=0
    visuimg-=visuimg.min()
    visuimg/=visuimg.max()  
    return visuimg




f       = '100075.jpg'
f       = '69015.jpg'
#f       = "/media/tbeier/GSP1RMCPRFR/iso.03530.png"
img     = vigra.impex.readImage(f)


print(img.shape)

if(img.shape[2]==1):
    img    = numpy.concatenate([img]*3,axis=2)
    imgLab = img
    imgLab = vigra.taggedView(imgLab,'xyc')
else:
    imgLab = vigra.colors.transform_RGB2Lab(img)
sigma   = 1.0


imgLab-=imgLab.min()
imgLab/=imgLab.max()
imgLab*=255

img-=img.min()
img/=img.max()
img*=255

print(imgLab.shape)


print("interpolate image")
imgLabSmall = imgLab

# make a few edge weights

gradmag = numpy.squeeze(vigra.filters.gaussianGradientMagnitude(imgLabSmall,sigma))
hessian = numpy.squeeze(vigra.filters.hessianOfGaussianEigenvalues(imgLabSmall[:,:,0],sigma))[:,:,0]
hessian-=hessian.min()
raw     = 256-imgLabSmall[:,:,0].copy()
gridGraph  = vigraph.gridGraph(imgLab.shape[:2],False)  



weights  = makeWeights(3.0)


pathFinder = vigraph.ShortestPathPathDijkstra(gridGraph)


visuimg =img.copy()
ax = plt.gca()
fig = plt.gcf()
visuimg-=visuimg.min()
visuimg/=visuimg.max()
implot = ax.imshow(numpy.swapaxes(visuimg,0,1),cmap='gray')

clickList=[]
frozen = False


axslider   = plt.axes([0.0, 0.00, 0.4, 0.075])
axfreeze   = plt.axes([0.6, 0.00, 0.1, 0.075])
axunfreeze = plt.axes([0.8, 0.00, 0.1, 0.075])
bfreeze    = Button(axfreeze, 'freeze')
bunfreeze  = Button(axunfreeze, 'unfrease and clear')
sgamma     = Slider(axslider, 'gamma', 0.01, 5.0, valinit=1.0)

def onclick(event):
    global clickList
    global weights
    global img
    if event.xdata != None and event.ydata != None:
        xRaw,yRaw = event.xdata,event.ydata
        if not frozen and xRaw >=0.0 and yRaw>=0.0 and xRaw<img.shape[0] and yRaw<img.shape[1]:
            x,y = int(math.floor(event.xdata)),int(math.floor(event.ydata))
            clickList.append((x,y))
            if len(clickList)==2:
                source = gridGraph.coordinateToNode(clickList[0])
                target = gridGraph.coordinateToNode(clickList[1])
                weights  = makeWeights(sgamma.val)
                #path = pathFinder.run(weights, source,target).path(pathType='coordinates')
                path = pathFinder.run(weights, source).path(pathType='coordinates',target=target)
                visuimg = makeVisuImage(path,img)
                implot.set_data(numpy.swapaxes(visuimg,0,1))
                plt.draw()
            

def freeze(event):
    global frozen
    frozen=True

def unfreeze(event):
    global frozen,clickList
    frozen=False
    clickList = []

def onslide(event):
    global img,gradmag,weights,clickList,sgamma
    weights  = makeWeights(sgamma.val)
    print("onslide",clickList)
    if len(clickList)>=2:
        print("we have  path")
        source = gridGraph.coordinateToNode(clickList[0])
        target = gridGraph.coordinateToNode(clickList[1])
        path = pathFinder.run(weights, source,target).path(pathType='coordinates')
        visuimg = makeVisuImage(path,img)
        implot.set_data(numpy.swapaxes(visuimg,0,1))
        plt.draw()


bfreeze.on_clicked(freeze)
bunfreeze.on_clicked(unfreeze)
sgamma.on_changed(onslide)

cid = fig.canvas.mpl_connect('button_press_event', onclick)
plt.show()



from __future__ import print_function

import vigra
from vigra import numpy
from matplotlib import pylab
from time import time
import multiprocessing

path = "69015.jpg"
#path = "12074.jpg"
path = "100075.jpg" 
path = "12003.jpg"
data = vigra.impex.readImage(path).astype(numpy.float32)

cpus = multiprocessing.cpu_count()

print("nCpus",cpus)

t0 =time()

#for c in range(3):
#    cimg=data[:,:,c]
#    cimg-=cimg.min()
#    cimg/=cimg.max()


iters = 10

#policy = vigra.filters.RatioPolicy(sigma=10.0, meanRatio=0.95, varRatio=0.5)
policy = vigra.filters.NormPolicy(sigma=50.0, meanDist=50, varRatio=0.5)
#data-=100.0
res = vigra.filters.nonLocalMean2d(data,policy=policy,searchRadius=5,patchRadius=1,nThreads=cpus+1,stepSize=2,verbose=True,sigmaMean=10.0)
for i in range(iters-1):
    res = vigra.filters.nonLocalMean2d(res,policy=policy,searchRadius=5,patchRadius=2,nThreads=cpus+1,stepSize=2,verbose=True,sigmaMean=10.0)
t1 = time()

res = vigra.taggedView(res,'xyc')
gma = vigra.filters.gaussianGradientMagnitude(res,4.0)
gmb = vigra.filters.gaussianGradientMagnitude(data,4.0)
#data+=100.0
print(t1-t0)
imgs  = [data,res,gma,gmb]

for img in imgs:
    for c in range(img.shape[2]):
        cimg=img[:,:,c]
        cimg-=cimg.min()
        cimg/=cimg.max()

f = pylab.figure()
for n, arr in enumerate(imgs):
    arr = arr.squeeze()
    f.add_subplot(1, len(imgs), n+1)
    pylab.imshow(arr.swapaxes(0,1))

pylab.title('denoised')
pylab.show()
